from pyxform.tests_v1.pyxform_test_case import PyxformTestCase

expected_xml_output = """
    <upload mediatype="osm/*" ref="/osm/osm_building">
      <label>Building</label>
      <tag key="name">
        <label>Name</label>
      </tag>
      <tag key="addr:city">
        <label>City</label>
      </tag>
    </upload>"""

class OSMWidgetsTest(PyxformTestCase):
    def test_osm_type(self):
        self.assertPyxformXform(
            name="osm",
            run_odk_validate=True,
            md="""
            | survey |                   |              |          |
            |        | type              |   name       | label    |
            |        | osm               | osm_road     | Road     |
            |        | osm building_tags | osm_building | Building |
            | osm    |                   |              |          |
            |        | list name         |  name        | label    |
            |        | building_tags     | name         | Name     |
            |        | building_tags     | addr:city    | City     |

            """,
            xml__contains=[expected_xml_output],
        )

    def test_osm_type_with_list_underscore_name(self):
        self.assertPyxformXform(
            name="osm",
            md="""
            | survey |                   |              |          |
            |        | type              |   name       | label    |
            |        | osm               | osm_road     | Road     |
            |        | osm building_tags | osm_building | Building |
            | osm    |                   |              |          |
            |        | list_name         |  name        | label    |
            |        | building_tags     | name         | Name     |
            |        | building_tags     | addr:city    | City     |

            """,
            xml__contains=[expected_xml_output],
        )



from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class XmlStructureTest(PyxformTestCase):
    def test_xml_structure(self):
        # re: https://github.com/XLSForm/pyxform/issues/14
        self.assertPyxformXform(
            name="testxmlstructure",
            md="""
            | survey |      |      |       |
            |        | type | name | label |
            |        | note | q    | Q     |
            """,
            xml__contains=[
                'xmlns="http://www.w3.org/2002/xforms"',
                'xmlns:h="http://www.w3.org/1999/xhtml"',
                'xmlns:jr="http://openrosa.org/javarosa"',
                'xmlns:orx="http://openrosa.org/xforms"',
                'xmlns:xsd="http://www.w3.org/2001/XMLSchema"',
            ],
        )



from xml.dom.minidom import Text, Element, parseString
import re
import codecs
import json
import copy
import unicodecsv as csv
import xlrd

try:
    unicode("str")
except NameError:
    unicode = str
    basestring = str
    unichr = chr
else:
    unicode = unicode
    basestring = basestring
    unichr = unichr

SEP = "_"

# http://www.w3.org/TR/REC-xml/
TAG_START_CHAR = r"[a-zA-Z:_]"
TAG_CHAR = r"[a-zA-Z:_0-9\-.]"
XFORM_TAG_REGEXP = "%(start)s%(char)s*" % {
    "start": TAG_START_CHAR,
    "char": TAG_CHAR
    }


class DetachableElement(Element):
    """
    Element classes are not meant to be instantiated directly.   This
    restriction was not strictly enforced in Python 2, but is effectively
    enforced in Python 3 via http://bugs.python.org/issue15290.

    A simple workaround (for now) is to set an extra attribute on the Element
    class.  The long term fix will probably be to rewrite node() to use
    document.createElement rather than Element.
    """
    def __init__(self, *args, **kwargs):
        Element.__init__(self, *args, **kwargs)
        self.ownerDocument = None


class PatchedText(Text):

    def writexml(self, writer, indent="", addindent="", newl=""):
        """Same as original but no replacing double quotes with '&quot;'."""
        data = "%s%s%s" % (indent, self.data, newl)
        if data:
            data = data.replace("&", "&amp;").replace("<", "&lt;"). \
                replace(">", "&gt;")
        writer.write(data)


def is_valid_xml_tag(tag):
    """
    Use a regex to see if there are any invalid characters (i.e. spaces).
    """
    return re.search(r"^" + XFORM_TAG_REGEXP + r"$", tag)


def node(*args, **kwargs):
    """
    args[0] -- a XML tag
    args[1:] -- an array of children to append to the newly created node
            or if a unicode arg is supplied it will be used to make a text node
    kwargs -- attributes
    returns a xml.dom.minidom.Element
    """
    blocked_attributes = ['tag']
    tag = args[0] if len(args) > 0 else kwargs['tag']
    args = args[1:]
    result = DetachableElement(tag)
    unicode_args = [u for u in args if type(u) == unicode]
    assert len(unicode_args) <= 1
    parsed_string = False
    # kwargs is an xml attribute dictionary,
    # here we convert it to a xml.dom.minidom.Element
    for k, v in iter(kwargs.items()):
        if k in blocked_attributes:
            continue
        if k == 'toParseString':
            if v is True and len(unicode_args) == 1:
                parsed_string = True
                # Add this header string so parseString can be used?
                s = u'<?xml version="1.0" ?><'+tag+'>' + unicode_args[0]\
                    + u'</'+tag+'>'
                parsed_node = parseString(s.encode("utf-8")).documentElement
                # Move node's children to the result Element
                # discarding node's root
                for child in parsed_node.childNodes:
                    result.appendChild(copy.deepcopy(child))
        else:
            result.setAttribute(k, v)

    if len(unicode_args) == 1 and not parsed_string:
        text_node = PatchedText()
        text_node.data = unicode_args[0]
        result.appendChild(text_node)
    for n in args:
        if type(n) == int or type(n) == float or type(n) == bytes:
            text_node = PatchedText()
            text_node.data = unicode(n)
            result.appendChild(text_node)
        elif type(n) is not unicode:
            try:
                result.appendChild(n)
            except:
                raise Exception(type(n), n)
    return result


def get_pyobj_from_json(str_or_path):
    """
    This function takes either a json string or a path to a json file,
    it loads the json into memory and returns the corresponding Python
    object.
    """
    try:
        # see if treating str_or_path as a path works
        fp = codecs.open(str_or_path, mode="r", encoding="utf-8")
        doc = json.load(fp, encoding="utf-8")
    except:
        # if it doesn't work load the text
        doc = json.loads(str_or_path)
    return doc


def flatten(li):
    for subli in li:
        for it in subli:
            yield it


def sheet_to_csv(workbook_path, csv_path, sheet_name):
    wb = xlrd.open_workbook(workbook_path)
    try:
        sheet = wb.sheet_by_name(sheet_name)
    except xlrd.biffh.XLRDError:
        return False
    if not sheet or sheet.nrows < 2:
        return False
    with open(csv_path, 'wb') as f:
        writer = csv.writer(f, quoting=csv.QUOTE_ALL)
        mask = [v and len(v.strip()) > 0 for v in sheet.row_values(0)]
        for r in range(sheet.nrows):
            writer.writerow(
                [v for v, m in zip(sheet.row_values(r), mask) if m])
    return True


def has_external_choices(json_struct):
    """
    Returns true if a select one external prompt is used in the survey.
    """
    if isinstance(json_struct, dict):
        for k, v in json_struct.items():
            if k == u"type" and isinstance(v, basestring) \
                    and v.startswith(u"select one external"):
                return True
            elif has_external_choices(v):
                return True
    elif isinstance(json_struct, list):
        for v in json_struct:
            if has_external_choices(v):
                return True
    return False



from pyxform.xform_instance_parser import parse_xform_instance


class SurveyInstance(object):
    def __init__(self, survey_object, **kwargs):
        self._survey = survey_object
        self.kwargs = kwargs  # not sure what might be passed to this

        # does the survey object provide a way to get the key dicts?
        self._keys = [c.name for c in self._survey.children]

        self._name = self._survey.name
        self._id = self._survey.id_string

        # get xpaths
        #  - prep for xpaths.
        self._survey.xml()
        self._xpaths = self._survey._xpath.values()

        # see "answers(self):" below for explanation of this dict
        self._answers = {}
        self._orphan_answers = {}

    def keys(self):
        return self._keys

    def xpaths(self):
        # originally thought of having this method get the xpath stuff
        # but survey doesn't like when xml() is called multiple times.
        return self._xpaths

    def answer(self, name=None, value=None):
        if name is None:
            raise Exception("In answering, name must be given")

        # ahh. this is horrible, but we need the xpath dict in survey
        # to be up-to-date ...maybe
        # self._survey.xml()

        if name in self._survey._xpath.keys():
            self._answers[name] = value
        else:
            self._orphan_answers[name] = value

    def to_json_dict(self):
        children = []
        for k, v in self._answers.items():
            children.append({'node_name': k, 'value': v})
        return {
            'node_name': self._name,
            'id': self._id,
            'children': children
        }

    def to_xml(self):
        """
        A horrible way to do this, but it works (until we need the attributes
         pumped out in order, etc)
        """
        open_str = """<?xml version='1.0' ?><%s id="%s">""" % (
            self._name, self._id)
        close_str = """</%s>""" % self._name
        vals = ""
        for k, v in self._answers.items():
            vals += "<%s>%s</%s>" % (k, str(v), k)

        output = open_str + vals + close_str
        return output

    def answers(self):
        """
        This returns "_answers", which is a dict with the key-value
        responses for this given instance. This could be pumped to xml
        or returned as a dict for maximum convenience (i.e. testing.)
        """
        return self._answers

    def import_from_xml(self, xml_string_or_filename):
        import os.path
        if os.path.isfile(xml_string_or_filename):
            xml_str = open(xml_string_or_filename).read()
        else:
            xml_str = xml_string_or_filename
        key_val_dict = parse_xform_instance(xml_str)
        for k, v in key_val_dict.items():
            self.answer(name=k, value=v)

    def __unicode__(self):
        orphan_count = len(self._orphan_answers.keys())
        placed_count = len(self._answers.keys())
        answer_count = orphan_count + placed_count
        return "<Instance (%d answers: %d placed. %d orphans)>" % (
            answer_count, placed_count, orphan_count)



from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class GeoWidgetsTest(PyxformTestCase):
    def test_gps_type(self):
        self.assertPyxformXform(
            name="geo",
            md="""
            | survey |      |          |       |
            |        | type |   name   | label |
            |        | gps  | location | GPS   |
            """,
            xml__contains=['geopoint'],
        )

    def test_gps_alias(self):
        self.assertPyxformXform(
            name="geo_alias",
            md="""
            | survey |          |          |       |
            |        | type     | name     | label |
            |        | geopoint | location | GPS   |
            """,
            xml__contains=['geopoint'],
        )

    def test_geo_widgets_types(self):
        """
        this test could be broken into multiple smaller tests.
        """
        self.assertPyxformXform(
            name="geos",
            md="""
            | survey |              |            |                   |
            |        | type         | name       | label             |
            |        | begin_repeat | repeat     |                   |
            |        | geopoint     | point      | Record Geopoint   |
            |        | note         | point_note | Point ${point}    |
            |        | geotrace     | trace      | Record a Geotrace |
            |        | note         | trace_note | Trace: ${trace}   |
            |        | geoshape     | shape      | Record a Geoshape |
            |        | note         | shape_note | Shape: ${shape}   |
            |        | end_repeat   |            |                   |
            """,
            xml__contains=[
                '<point/>',
                '<point_note/>',

                '<trace/>',
                '<trace_note/>',

                '<shape/>',
                '<shape_note/>',

                '<bind nodeset="/geos/repeat/point" type="geopoint"/>',
                '<bind nodeset="/geos/repeat/point_note" readonly="true()" '
                'type="string"/>',

                '<bind nodeset="/geos/repeat/trace" type="geotrace"/>',
                '<bind nodeset="/geos/repeat/trace_note" readonly="true()" '
                'type="string"/>',

                '<bind nodeset="/geos/repeat/shape" type="geoshape"/>',
                '<bind nodeset="/geos/repeat/shape_note" readonly="true()" '
                'type="string"/>',
                ],
            )



import json
from pyxform.utils import is_valid_xml_tag, node, unicode
from pyxform.xls2json import print_pyobj_to_json
from pyxform.question_type_dictionary import QUESTION_TYPE_DICT
from pyxform.errors import PyXFormError


def _overlay(over, under):
    if type(under) == dict:
        result = under.copy()
        result.update(over)
        return result
    return over if over else under


class SurveyElement(dict):
    """
    SurveyElement is the base class we'll looks for the following keys
    in kwargs: name, label, hint, type, bind, control, parent,
    children, and question_type_dictionary.
    """

    # the following are important keys for the underlying dict that
    # describes this survey element
    FIELDS = {
        u"name": unicode,
        u"sms_field": unicode,
        u"sms_option": unicode,
        u"label": unicode,
        u"hint": unicode,
        u"default": unicode,
        u"type": unicode,
        u"appearance": unicode,
        u"parameters": unicode,
        u"intent": unicode,
        u"jr:count": unicode,
        u"bind": dict,
        u"instance": dict,
        u"control": dict,
        u"media": dict,
        # this node will also have a parent and children, like a tree!
        u"parent": lambda: None,
        u"children": list,
        u"itemset": unicode,
        u"choice_filter": unicode,
        u"query": unicode,
        u"autoplay": unicode,
        u"flat": lambda: False,
    }

    def _default(self):
        # TODO: need way to override question type dictionary
        defaults = QUESTION_TYPE_DICT
        return defaults.get(self.get(u"type"), {})

    def __getattr__(self, key):
        """
        Get attributes from FIELDS rather than the class.
        """
        if key in self.FIELDS:
            question_type_dict = self._default()
            under = question_type_dict.get(key, None)
            over = self.get(key)
            if not under:
                return over
            return _overlay(over, under)
        raise AttributeError(key)

    def __name__(self):
        return "SurveyElement"

    def __setattr__(self, key, value):
        self[key] = value

    def __init__(self, **kwargs):
        for key, default in self.FIELDS.items():
            self[key] = kwargs.get(key, default())
        self._link_children()

        # Create a space label for unlabeled elements with the label
        # appearance tag. # This is because such elements are used to label the
        # options for selects in a field-list and might want blank labels for
        # themselves.
        if self.control.get(u"appearance") == u"label" and not self.label:
            self[u"label"] = u" "

    def _link_children(self):
        for child in self.children:
            child.parent = self

    def add_child(self, child):
        self.children.append(child)
        child.parent = self

    def add_children(self, children):
        if type(children) == list:
            for child in children:
                self.add_child(child)
        else:
            self.add_child(children)

    binding_conversions = {
        "yes": "true()",
        "Yes": "true()",
        "YES": "true()",
        "true": "true()",
        "True": "true()",
        "TRUE": "true()",
        "no": "false()",
        "No": "false()",
        "NO": "false()",
        "false": "false()",
        "False": "false()",
        "FALSE": "false()"
    }

    # Supported media types for attaching to questions
    SUPPORTED_MEDIA = [
        "image",
        "audio",
        "video"
    ]

    def validate(self):
        if not is_valid_xml_tag(self.name):
            msg = "The name '%s' is an invalid xml tag. Names must begin with" \
                  " a letter, colon, or underscore, subsequent characters can" \
                  " include numbers, dashes, and periods." % self.name
            raise PyXFormError(msg)

    # TODO: Make sure renaming this doesn't cause any problems
    def iter_descendants(self):
        """
        A survey_element is a dictionary of survey_elements
        This method does a preorder traversal over them.
        For the time being this survery_element is included among its
        descendants
        """
        # it really seems like this method should not yield self
        yield self
        for e in self.children:
            for f in e.iter_descendants():
                yield f

    def get_lineage(self):
        """
        Return a the list [root, ..., self._parent, self]
        """
        result = [self]
        current_element = self
        while current_element.parent:
            current_element = current_element.parent
            result = [current_element] + result
        # For some reason the root element has a True flat property...
        output = [result[0]]
        for item in result[1:]:
            if not item.get(u"flat"):
                output.append(item)
        return output

    def get_root(self):
        return self.get_lineage()[0]

    def get_xpath(self):
        """
        Return the xpath of this survey element.
        """
        return u"/".join([u""] + [n.name for n in self.get_lineage()])

    def get_abbreviated_xpath(self):
        lineage = self.get_lineage()
        if len(lineage) >= 2:
            return u"/".join([unicode(n.name) for n in lineage[1:]])
        else:
            return lineage[0].name

    def to_json_dict(self):
        """
        Create a dict copy of this survey element by removing inappropriate
        attributes and converting its children to dicts
        """
        self.validate()
        result = self.copy()
        to_delete = [u"parent", u"question_type_dictionary", u"_created"]
        for key in to_delete:
            if key in result:
                del result[key]
        children = result.pop(u"children")
        result[u"children"] = []
        for child in children:
            result[u"children"].append(child.to_json_dict())
        # remove any keys with empty values
        for k, v in list(result.items()):
            if not v:
                del result[k]

        return result

    def to_json(self):
        return json.dumps(self.to_json_dict())

    def json_dump(self, path=""):
        if not path:
            path = self.name + ".json"
        print_pyobj_to_json(self.to_json_dict(), path)

    def __eq__(self, y):
        return hasattr(y, 'to_json_dict') and callable(y.to_json_dict) and \
               self.to_json_dict() == y.to_json_dict()

    def _translation_path(self, display_element):
        return self.get_xpath() + ":" + display_element

    def get_translations(self, default_language):
        """
        Returns translations used by this element so they can be included in
        the <itext> block. @see survey._setup_translations
        """
        bind_dict = self.get(u'bind')
        if bind_dict and type(bind_dict) is dict:
            constraint_msg = bind_dict.get(u'jr:constraintMsg')
            if type(constraint_msg) is dict:
                for lang, text in constraint_msg.items():
                    yield {
                        'path': self._translation_path(u'jr:constraintMsg'),
                        'lang': lang,
                        'text': text
                    }
            required_msg = bind_dict.get(u'jr:requiredMsg')
            if type(required_msg) is dict:
                for lang, text in required_msg.items():
                    yield {
                        'path': self._translation_path(u'jr:requiredMsg'),
                        'lang': lang,
                        'text': text
                    }
            no_app_error_string = bind_dict.get(u'jr:noAppErrorString')
            if type(no_app_error_string) is dict:
                for lang, text in no_app_error_string.items():
                    yield {
                        'path': self._translation_path(u'jr:noAppErrorString'),
                        'lang': lang,
                        'text': text
                    }

        for display_element in [u'label', u'hint']:
            label_or_hint = self[display_element]

            if display_element is u'label' \
                    and self.needs_itext_ref() \
                    and type(label_or_hint) is not dict \
                    and label_or_hint:
                label_or_hint = {default_language: label_or_hint}

            if type(label_or_hint) is dict:
                for lang, text in label_or_hint.items():
                    yield {
                        'display_element': display_element,  # Not used
                        'path': self._translation_path(display_element),
                        'element': self,  # Not used
                        'lang': lang,
                        'text': text,
                    }

    def get_media_keys(self):
        """
        @deprected
        I'm leaving this in just in case it has outside references.
        """
        return {
            u"media": u"%s:media" % self.get_xpath()
        }

    def needs_itext_ref(self):
        return type(self.label) is dict or (
            type(self.media) is dict and len(self.media) > 0)

    # XML generating functions, these probably need to be moved around.
    def xml_label(self):
        if self.needs_itext_ref():
            # If there is a dictionary label, or non-empty media dict,
            # then we need to make a label with an itext ref
            ref = "jr:itext('%s')" % self._translation_path(u"label")
            return node(u"label", ref=ref)
        else:
            survey = self.get_root()
            label, output_inserted = survey.insert_output_values(self.label)
            return node(u"label", label, toParseString=output_inserted)

    def xml_hint(self):
        if type(self.hint) == dict:
            path = self._translation_path("hint")
            return node(u"hint", ref="jr:itext('%s')" % path)
        else:
            hint, output_inserted = self.get_root().insert_output_values(
                self.hint)
            return node(u"hint", hint, toParseString=output_inserted)

    def xml_label_and_hint(self):
        """
        Return a list containing one node for the label and if there
        is a hint one node for the hint.
        """
        result = []
        if self.label or self.media:
            result.append(self.xml_label())
        if self.hint:
            result.append(self.xml_hint())

        if len(result) == 0:
            msg = "The survey element named '%s' " \
                  "has no label or hint." % self.name
            raise PyXFormError(msg)

        return result

    def xml_binding(self):
        """
        Return the binding for this survey element.
        """
        survey = self.get_root()
        bind_dict = self.bind.copy()
        if self.get('flat'):
            # Don't generate bind element for flat groups.
            return None
        if bind_dict:
            for k, v in bind_dict.items():
                # I think all the binding conversions should be happening on
                # the xls2json side.
                if hashable(v) and v in self.binding_conversions:
                    v = self.binding_conversions[v]
                if k == u'jr:constraintMsg' and type(v) is dict:
                    v = "jr:itext('%s')" % self._translation_path(
                        u'jr:constraintMsg')
                if k == u'jr:requiredMsg' and type(v) is dict:
                    v = "jr:itext('%s')" % self._translation_path(
                        u'jr:requiredMsg')
                if k == u'jr:noAppErrorString' and type(v) is dict:
                    v = "jr:itext('%s')" % self._translation_path(
                        u'jr:noAppErrorString')
                bind_dict[k] = survey.insert_xpaths(v)
            return node(u"bind", nodeset=self.get_xpath(), **bind_dict)
        return None

    def xml_bindings(self):
        """
        Return a list of bindings for this node and all its descendants.
        """
        result = []
        for e in self.iter_descendants():
            xml_binding = e.xml_binding()
            if xml_binding is not None:
                result.append(xml_binding)
        return result

    def xml_control(self):
        """
        The control depends on what type of question we're asking, it
        doesn't make sense to implement here in the base class.
        """
        raise Exception("Control not implemented")


def hashable(v):
    """Determine whether `v` can be hashed."""
    try:
        hash(v)
    except TypeError:
        return False
    return True



from __future__ import print_function

import argparse
import json
import os

from pyxform import builder, xls2json
from pyxform.utils import has_external_choices, sheet_to_csv
"""
xls2xform converts properly formatted Excel documents into XForms for
use with ODK Collect.
"""


def xls2xform_convert(xlsform_path, xform_path, validate=True, pretty_print=True):
    warnings = []

    json_survey = xls2json.parse_file_to_json(xlsform_path, warnings=warnings)
    survey = builder.create_survey_element_from_dict(json_survey)
    # Setting validate to false will cause the form not to be processed by
    # ODK Validate.
    # This may be desirable since ODK Validate requires launching a subprocess
    # that runs some java code.
    survey.print_xform_to_file(
        xform_path, validate=validate,
        pretty_print=pretty_print, warnings=warnings)
    output_dir = os.path.split(xform_path)[0]
    if has_external_choices(json_survey):
        itemsets_csv = os.path.join(output_dir, "itemsets.csv")
        choices_exported = sheet_to_csv(xlsform_path, itemsets_csv,
                                        "external_choices")
        if not choices_exported:
            warnings.append("Could not export itemsets.csv, perhaps the "
                            "external choices sheet is missing.")
        else:
            print('External choices csv is located at:', itemsets_csv)
    return warnings


def _create_parser():
    """
    Parse command line arguments.
    """
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "path_to_XLSForm",
        help="Path to the Excel XSLX file with the XLSForm definition.")
    parser.add_argument("output_path", help="Path to save the output to.")
    parser.add_argument(
        "--json",
        action="store_true",
        help="Capture everything and report in JSON format.")
    parser.add_argument(
        "--skip_validate",
        action="store_false",
        default=True,
        help="Skip default running of ODK Validate on the output XForm XML.")
    parser.add_argument(
        "--no_pretty_print",
        action="store_false",
        default=True,
        help="Print XML forms with collapsed whitespace instead of pretty-printed.")
    return parser


def main_cli():
    parser = _create_parser()
    args = parser.parse_args()

    if args.json:
        # Store everything in a list just in case the user wants to output
        # as a JSON encoded string.
        response = {'code': None, 'message': None, 'warnings': []}

        try:
            response['warnings'] = xls2xform_convert(
                args.path_to_XLSForm, args.output_path, args.skip_validate, args.no_pretty_print)

            response['code'] = 100
            response['message'] = "Ok!"

            if response['warnings']:
                response['code'] = 101
                response['message'] = 'Ok with warnings.'

        except Exception as e:
            # Catch the exception by default.
            response['code'] = 999
            response['message'] = str(e)

        print(json.dumps(response))
    else:
        warnings = xls2xform_convert(args.path_to_XLSForm, args.output_path,
                                     args.skip_validate, args.no_pretty_print)
        if len(warnings) > 0:
            print("Warnings:")
        for w in warnings:
            print(w)
        print('Conversion complete!')


if __name__ == '__main__':
    main_cli()



import os
import glob
from pyxform import utils

from pyxform.xls2json import SurveyReader


def _section_name(path_or_file_name):
    directory, filename = os.path.split(path_or_file_name)
    section_name, extension = os.path.splitext(filename)
    return section_name


def load_file_to_dict(path):
    """
    Takes a file path and loads it into a nested json dict
    following the format in json_form_schema.json
    The file may be a xls file or json file.
    If it is xls it is converted using xls2json.
    """
    if path.endswith(".json"):
        name = _section_name(path)
        return name, utils.get_pyobj_from_json(path)
    else:
        name = _section_name(path)
        excel_reader = SurveyReader(path)
        return name, excel_reader.to_json_dict()


def collect_compatible_files_in_directory(directory):
    """
    create a giant dict out of all the spreadsheets and json forms
    in the given directory
    """
    available_files = glob.glob(os.path.join(directory, "*.xls")) + \
        glob.glob(os.path.join(directory, "*.json"))

    return dict([load_file_to_dict(f) for f in available_files])



import re
import xml.etree.ElementTree as ETree
from unittest import TestCase
from pyxform.builder import SurveyElementBuilder, create_survey_from_xls
from pyxform.xls2json import print_pyobj_to_json
from pyxform import Survey, InputQuestion
from pyxform.errors import PyXFormError
from pyxform.tests import utils
import os

FIXTURE_FILETYPE = "xls"


class BuilderTests(TestCase):
    maxDiff = None

#   Moving to spec tests
#    def test_new_widgets(self):
#        survey = utils.build_survey('widgets.xls')
#        path = utils.path_to_text_fixture('widgets.xml')
#        survey.to_xml
#        with open(path) as f:
#            expected = ETree.fromstring(survey.to_xml())
#            result = ETree.fromstring(f.read())
#            self.assertTrue(xml_compare(expected, result))

    def test_unknown_question_type(self):
        survey = utils.build_survey('unknown_question_type.xls')
        self.assertRaises(
            PyXFormError,
            survey.to_xml
        )

    def test_uniqueness_of_section_names(self):
        # Looking at the xls file, I think this test might be broken.
        survey = utils.build_survey('group_names_must_be_unique.xls')
        self.assertRaises(
            Exception,
            survey.to_xml
        )

    def setUp(self):
        self.this_directory = os.path.dirname(__file__)
        survey_out = Survey(
            name=u"age",
            sms_keyword=u"age",
            type=u"survey"
        )
        question = InputQuestion(name=u"age")
        question.type = u"integer"
        question.label = u"How old are you?"
        survey_out.add_child(question)
        self.survey_out_dict = survey_out.to_json_dict()
        print_pyobj_to_json(self.survey_out_dict,
                            utils.path_to_text_fixture("how_old_are_you.json"))

    def test_create_from_file_object(self):
        path = utils.path_to_text_fixture('yes_or_no_question.xls')
        with open(path, 'rb') as f:
            create_survey_from_xls(f)

    def tearDown(self):
        fixture_path = utils.path_to_text_fixture("how_old_are_you.json")
        if os.path.exists(fixture_path):
            os.remove(fixture_path)

    def test_create_table_from_dict(self):
        d = {
            u"type": u"loop",
            u"name": u"my_loop",
            u"label": {u"English": u"My Loop"},
            u"columns": [
                {
                    u"name": u"col1",
                    u"label": {u"English": u"column 1"},
                },
                {
                    u"name": u"col2",
                    u"label": {u"English": u"column 2"},
                },
            ],
            u"children": [
                {
                    u"type": u"integer",
                    u"name": u"count",
                    u"label": {
                        u"English": u"How many are there in this group?"
                    }
                },
            ]
        }
        builder = SurveyElementBuilder()
        g = builder.create_survey_element_from_dict(d)

        expected_dict = {
            u'name': u'my_loop',
            u'label': {u'English': u'My Loop'},
            u'type': u'group',
            u'children': [
                {
                    u'name': u'col1',
                    u'label': {u'English': u'column 1'},
                    u'type': u'group',
                    u'children': [
                        {
                            u'name': u'count',
                            u'label': {
                                u'English':
                                u'How many are there in this group?'
                            },
                            u'type': u'integer'
                        }
                    ]
                },
                {
                    u'name': u'col2',
                    u'label': {u'English': u'column 2'},
                    u'type': u'group',
                    u'children': [
                        {
                            u'name': u'count',
                            u'label': {
                                u'English':
                                u'How many are there in this group?'
                            },
                            u'type': u'integer'
                        }
                    ]
                }
            ]
        }

        self.assertEqual(g.to_json_dict(), expected_dict)

    def test_specify_other(self):
        survey = utils.create_survey_from_fixture("specify_other",
                                                  filetype=FIXTURE_FILETYPE)
        expected_dict = {
            u'name': u'specify_other',
            u'type': u'survey',
            u'title': u'specify_other',
            u'default_language': u'default',
            u'id_string': u'specify_other',
            u'sms_keyword': u'specify_other',
            u'children': [
                {
                    u'name': u'sex',
                    u'label': {u'English': u'What sex are you?'},
                    u'type': u'select one',
                    u'children': [
                        # TODO Change to choices (there is stuff in the
                        # json2xform half that will need to change)
                        {
                            u'name': u'male',
                            u'label': {u'English': u'Male'}
                        },
                        {
                            u'name': u'female',
                            u'label': {u'English': u'Female'}
                        },
                        {
                            u'name': u'other',
                            u'label': u'Other'
                        }
                    ]
                },
                {
                    u'name': u'sex_other',
                    u'bind': {u'relevant': u"selected(../sex, 'other')"},
                    u'label': u'Specify other.',
                    u'type': u'text'},
                {
                    u'children': [
                        {
                            u'bind': {
                                'calculate': "concat('uuid:', uuid())",
                                'readonly': 'true()'
                            },
                            u'name': 'instanceID',
                            u'type': 'calculate'
                        }
                    ],
                    u'control': {
                        'bodyless': True
                    },
                    u'name': 'meta',
                    u'type': u'group'
                }
            ]
        }
        self.maxDiff = None
        self.assertEqual(survey.to_json_dict(), expected_dict)

    def test_select_one_question_with_identical_choice_name(self):
        """
        testing to make sure that select ones whose choice names are the same
        as the name of the select one get compiled.
        """
        survey = utils.create_survey_from_fixture(
            "choice_name_same_as_select_name", filetype=FIXTURE_FILETYPE)
        expected_dict = {
            u'name': u'choice_name_same_as_select_name',
            u'title': u'choice_name_same_as_select_name',
            u'sms_keyword': u'choice_name_same_as_select_name',
            u'default_language': u'default',
            u'id_string': u'choice_name_same_as_select_name',
            u'type': u'survey',
            u'children':  [
                {
                    u'children':  [
                        {
                            u'name': u'zone',
                            u'label': u'Zone'
                        }
                    ],
                    u'type': u'select one',
                    u'name': u'zone',
                    u'label': u'Zone',
                },
                {u'children': [
                    {
                        u'bind': {
                            'calculate': "concat('uuid:', uuid())",
                            'readonly': 'true()'
                        },
                        u'name': 'instanceID',
                        u'type': 'calculate'
                    }
                ],
                    u'control': {
                        'bodyless': True
                    },
                    u'name': 'meta',
                    u'type': u'group'
                }
            ]
        }
        self.maxDiff = None
        self.assertEqual(survey.to_json_dict(), expected_dict)

    def test_loop(self):
        survey = utils.create_survey_from_fixture(
            "loop", filetype=FIXTURE_FILETYPE)
        expected_dict = {
            u'name': u'loop',
            u'id_string': u'loop',
            u'sms_keyword': u'loop',
            u'title': u'loop',
            u'type': u'survey',
            u'default_language': u'default',
            u'children': [
                {
                    u'name': u'available_toilet_types',
                    u'label': {
                        u'english':
                        u'What type of toilets are on the premises?'
                    },
                    u'type': u'select all that apply',
                    u'children': [
                        {
                            u'name': u'pit_latrine_with_slab',
                            u'label': {u'english': u'Pit latrine with slab'}
                        },
                        {
                            u'name': u'open_pit_latrine',
                            u'label': {
                                u'english':
                                u'Pit latrine without slab/open pit'
                            }
                        },
                        {
                            u'name': u'bucket_system',
                            u'label': {u'english': u'Bucket system'}
                        },
                        # Removing this because select alls shouldn't need
                        # an explicit none option
                        # {
                        #    u'name': u'none',
                        #    u'label': u'None',
                        #    },
                        {
                            u'name': u'other',
                            u'label': u'Other'
                        },
                    ]
                },

                {
                    u'name': u'available_toilet_types_other',
                    u'bind': {
                        u'relevant':
                        u"selected(../available_toilet_types, 'other')"
                    },
                    u'label': u'Specify other.',
                    u'type': u'text'
                },
                {
                    u'name': u'loop_toilet_types',
                    u'type': u'group',
                    u'children': [
                        {
                            u'name': u'pit_latrine_with_slab',
                            u'label': {u'english': u'Pit latrine with slab'},
                            u'type': u'group',
                            u'children': [
                                {
                                    u'name': u'number',
                                    u'label': {
                                        u'english':
                                        u'How many Pit latrine with slab are'
                                        u' on the premises?'
                                    },
                                    u'type': u'integer'
                                }]},
                        {
                            u'name': u'open_pit_latrine',
                            u'label': {
                                u'english':
                                u'Pit latrine without slab/open pit'
                            },
                            u'type': u'group',
                            u'children': [
                                {
                                    u'name': u'number',
                                    u'label': {
                                        u'english':
                                        u'How many Pit latrine without '
                                        u'slab/open pit are on the premises?'
                                    },
                                    u'type': u'integer'
                                }
                            ]
                        },
                        {
                            u'name': u'bucket_system',
                            u'label': {u'english': u'Bucket system'},
                            u'type': u'group',
                            u'children': [
                                {
                                    u'name': u'number',
                                    u'label': {
                                        u'english':
                                        u'How many Bucket system are on the'
                                        u' premises?'},
                                    u'type': u'integer'
                                }
                            ]
                        }]},
                {
                    u'children': [
                        {
                            u'bind': {
                                'calculate': "concat('uuid:', uuid())",
                                'readonly': 'true()'
                            },
                            u'name': 'instanceID',
                            u'type': 'calculate'
                        }
                    ],
                    u'control': {
                        'bodyless': True
                    },
                    u'name': 'meta',
                    u'type': u'group'
                }]}
        self.maxDiff = None
        self.assertEqual(survey.to_json_dict(), expected_dict)

    def test_sms_columns(self):
        survey = utils.create_survey_from_fixture(
            "sms_info", filetype=FIXTURE_FILETYPE)
        expected_dict = {
            u'children': [{
                u'children': [
                    {
                        u'label': u'How old are you?',
                        u'name': u'age',
                        u'sms_field': u'q1',
                        u'type': u'integer'
                    },
                    {
                        u'children': [
                            {
                                u'label': u'no',
                                u'name': u'0',
                                u'sms_option': u'n'
                            },
                            {
                                u'label': u'yes',
                                u'name': u'1',
                                u'sms_option': u'y'
                            }],
                        u'label': u'Do you have any children?',
                        u'name': u'has_children',
                        u'sms_field': u'q2',
                        u'type': u'select one'
                    },
                    {
                        u'label': u"What's your birth day?",
                        u'name': u'bday',
                        u'sms_field': u'q3',
                        u'type': u'date'
                    },
                    {
                        u'label': u'What is your name?',
                        u'name': u'name',
                        u'sms_field': u'q4',
                        u'type': u'text'
                    }
                ],
                u'name': u'section1',
                u'sms_field': u'a',
                u'type': u'group'
            },
                {
                    u'children': [
                        {
                            u'label': u'May I take your picture?',
                            u'name': u'picture',
                            u'type': u'photo'
                        },
                        {
                            u'label':
                            u'Record your GPS coordinates.',
                            u'name': u'gps',
                            u'type': u'geopoint'
                        }
                    ],
                    u'name': u'medias',
                    u'sms_field': u'c',
                    u'type': u'group'
                }, {
                    u'children': [
                        {
                            u'children': [{
                                u'label': u'Mozilla Firefox',
                                u'name': u'firefox',
                                u'sms_option': u'ff'
                            },
                                {
                                    u'label': u'Google Chrome',
                                    u'name': u'chrome',
                                    u'sms_option': u'gc'
                                },
                                {
                                    u'label':
                                    u'Internet Explorer',
                                    u'name': u'ie',
                                    u'sms_option': u'ie'
                                },
                                {
                                    u'label': u'Safari',
                                    u'name': u'safari',
                                    u'sms_option': u'saf'
                                }
                            ],
                            u'label':
                            u'What web browsers do you use?',
                            u'name': u'web_browsers',
                            u'sms_field': u'q5',
                            u'type': u'select all that apply'
                        }
                    ],
                    u'name': u'browsers',
                    u'sms_field': u'b',
                    u'type': u'group'
                }, {
                    u'children': [{
                        u'label': u'Phone Number',
                        u'name': u'phone',
                        u'type': u'phonenumber'
                    }, {
                        u'label': u'Start DT',
                        u'name': u'start',
                        u'type': u'start'
                    }, {
                        u'label': u'End DT',
                        u'name': u'end',
                        u'type': u'end'
                    }, {
                        u'label': u'Send Day',
                        u'name': u'today',
                        u'type': u'today'
                    }, {
                        u'label': u'IMEI',
                        u'name': u'imei',
                        u'type': u'deviceid'
                    }, {
                        u'label': u'Hey!',
                        u'name': u'nope',
                        u'type': u'note'
                    }],
                    u'name': u'metadata',
                    u'sms_field': u'meta',
                    u'type': u'group'
                },
                {
                    u'children': [{
                        u'bind': {
                            'calculate': "concat('uuid:', uuid())",
                            'readonly': 'true()'
                        },
                        u'name': 'instanceID',
                        u'type': 'calculate'
                    }],
                    u'control': {'bodyless': True},
                    u'name': 'meta',
                    u'type': u'group'
                }],
            u'default_language': u'default',
            u'id_string': u'sms_info_form',
            u'name': u'sms_info',
            u'sms_allow_media': u'TRUE',
            u'sms_date_format': u'%Y-%m-%d',
            u'sms_datetime_format': u'%Y-%m-%d-%H:%M',
            u'sms_keyword': u'inf',
            u'sms_separator': u'+',
            u'title': u'SMS Example',
            u'type': u'survey'
        }
        self.assertEqual(survey.to_json_dict(), expected_dict)

    def test_style_column(self):
        survey = utils.create_survey_from_fixture(
            "style_settings", filetype=FIXTURE_FILETYPE)
        expected_dict = {
            u'children': [
                {
                    u'label': {u'english': u'What is your name?'},
                    u'name': u'your_name',
                    u'type': u'text'
                },
                {
                    u'label': {u'english': u'How many years old are you?'},
                    u'name': u'your_age',
                    u'type': u'integer'
                },
                {
                    u'children': [
                        {
                            u'bind': {
                                'calculate': "concat('uuid:', uuid())",
                                'readonly': 'true()'
                            },
                            u'name': 'instanceID',
                            u'type': 'calculate'
                        }
                    ],
                    u'control': {'bodyless': True},
                    u'name': 'meta',
                    u'type': u'group'
                }
            ],
            u'default_language': u'default',
            u'id_string': u'new_id',
            u'name': u'style_settings',
            u'sms_keyword': u'new_id',
            u'style': u'ltr',
            u'title': u'My Survey',
            u'type': u'survey',
        }
        self.assertEqual(survey.to_json_dict(), expected_dict)

    STRIP_NS_FROM_TAG_RE = re.compile(r'\{.+\}')

    def test_style_not_added_to_body_if_not_present(self):
        survey = utils.create_survey_from_fixture(
            "settings", filetype=FIXTURE_FILETYPE)
        xml = survey.to_xml()
        # find the body tag
        root_elm = ETree.fromstring(xml.encode('utf-8'))
        body_elms = list(filter(
            lambda e: self.STRIP_NS_FROM_TAG_RE.sub('', e.tag) == 'body',
            [c for c in root_elm.getchildren()]))
        self.assertEqual(len(body_elms), 1)
        self.assertIsNone(body_elms[0].get('class'))

    def test_style_added_to_body_if_present(self):
        survey = utils.create_survey_from_fixture(
            "style_settings", filetype=FIXTURE_FILETYPE)
        xml = survey.to_xml()
        # find the body tag
        root_elm = ETree.fromstring(xml.encode('utf-8'))
        body_elms = list(filter(
            lambda e: self.STRIP_NS_FROM_TAG_RE.sub('', e.tag) == 'body',
            [c for c in root_elm.getchildren()]))
        self.assertEqual(len(body_elms), 1)
        self.assertEqual(body_elms[0].get('class'), 'ltr')



"""
Testing simple cases for Xls2Json
"""
from unittest2 import TestCase
from pyxform.xls2json import SurveyReader
from pyxform.xls2json_backends import xls_to_dict, csv_to_dict
from pyxform.tests import utils
import os
import json
import codecs


# Nothing calls this AFAICT
def absolute_path(f, file_name):
    directory = os.path.dirname(f)
    return os.path.join(directory, file_name)


DIR = os.path.dirname(__file__)


class BasicXls2JsonApiTests(TestCase):
    maxDiff = None

    def test_simple_yes_or_no_question(self):
        filename = "yes_or_no_question.xls"
        path_to_excel_file = os.path.join(DIR, "example_xls", filename)
        # Get the xform output path:
        root_filename, ext = os.path.splitext(filename)
        output_path = os.path.join(DIR, "test_output", root_filename + ".json")
        expected_output_path = os.path.join(DIR, "test_expected_output",
                                            root_filename + ".json")
        x = SurveyReader(path_to_excel_file)
        x_results = x.to_json_dict()
        with codecs.open(output_path, mode="w", encoding="utf-8") as fp:
            json.dump(x_results, fp=fp, ensure_ascii=False, indent=4)
        # Compare with the expected output:
        with codecs.open(expected_output_path, 'rb',
                         encoding="utf-8") as expected_file:
            with codecs.open(output_path, 'rb',
                             encoding="utf-8") as actual_file:
                expected_json = json.load(expected_file)
                actual_json = json.load(actual_file)
                self.assertEqual(expected_json, actual_json)

    def test_hidden(self):
        x = SurveyReader(utils.path_to_text_fixture("hidden.xls"))
        x_results = x.to_json_dict()

        expected_dict = [
            {
                u'type': u'hidden',
                u'name': u'hidden_test'
            },
            {
                'children': [
                    {
                        'bind': {
                            'calculate': "concat('uuid:', uuid())",
                            'readonly': 'true()'
                        },
                        'name': 'instanceID',
                        'type': 'calculate'
                    }
                ],
                'control': {
                    'bodyless': True
                },
                'name': 'meta',
                'type': 'group'
            }
        ]
        self.assertEqual(x_results[u"children"], expected_dict)

    def test_gps(self):
        x = SurveyReader(utils.path_to_text_fixture("gps.xls"))

        expected_dict = [
            {
                u'type': u'gps', u'name': u'location', u'label': u'GPS'},
            {
                'children': [
                    {
                        'bind': {
                            'calculate': "concat('uuid:', uuid())",
                            'readonly': 'true()'
                        },
                        'name': 'instanceID',
                        'type': 'calculate'
                    }
                ],
                'control': {
                    'bodyless': True
                },
                'name': 'meta',
                'type': 'group'
            }]

        self.assertEqual(x.to_json_dict()[u"children"], expected_dict)

    def test_text_and_integer(self):
        x = SurveyReader(utils.path_to_text_fixture("text_and_integer.xls"))

        expected_dict = [
            {
                u'label': {
                    u'english': u'What is your name?'
                },
                u'type': u'text',
                u'name': u'your_name'
            },
            {
                u'label': {
                    u'english': u'How many years old are you?'
                },
                u'type': u'integer',
                u'name': u'your_age'
            },
            {
                u'children': [
                    {
                        u'bind': {
                            'calculate': "concat('uuid:', uuid())",
                            'readonly': 'true()'
                        },
                        u'name': 'instanceID',
                        u'type': 'calculate'
                    }
                ],
                u'control': {
                    'bodyless': True
                },
                u'name': 'meta',
                u'type': u'group'
            }
        ]

        self.assertEqual(x.to_json_dict()[u"children"], expected_dict)

    def test_table(self):
        filename = "simple_loop.xls"
        path_to_excel_file = os.path.join(DIR, "example_xls", filename)
        # Get the xform output path:
        root_filename, ext = os.path.splitext(filename)
        output_path = os.path.join(DIR, "test_output", root_filename + ".json")
        expected_output_path = os.path.join(DIR, "test_expected_output",
                                            root_filename + ".json")
        x = SurveyReader(path_to_excel_file)
        x_results = x.to_json_dict()
        with codecs.open(output_path, mode="w", encoding="utf-8") as fp:
            json.dump(x_results, fp=fp, ensure_ascii=False, indent=4)
        # Compare with the expected output:
        with codecs.open(expected_output_path, 'rb',
                         encoding="utf-8") as expected_file:
            with codecs.open(output_path, 'rb',
                             encoding="utf-8") as actual_file:
                expected_json = json.load(expected_file)
                actual_json = json.load(actual_file)
                self.assertEqual(expected_json, actual_json)


class CsvReaderEquivalencyTest(TestCase):
    def test_equivalency(self):
        equivalent_fixtures = ['group', 'loop',  # 'gps',
                               'specify_other', 'include', 'text_and_integer',
                               'include_json', 'yes_or_no_question']
        for fixture in equivalent_fixtures:
            xls_path = utils.path_to_text_fixture("%s.xls" % fixture)
            csv_path = utils.path_to_text_fixture("%s.csv" % fixture)
            xls_inp = xls_to_dict(xls_path)
            csv_inp = csv_to_dict(csv_path)
            self.maxDiff = None
            self.assertEqual(csv_inp, xls_inp)


class UnicodeCsvTest(TestCase):
    def test_a_unicode_csv_works(self):
        """
        Simply tests that xls2json_backends.csv_to_dict does not have a problem
        with a csv with unicode characters
        """
        utf_csv_path = utils.path_to_text_fixture("utf_csv.csv")
        dict_value = csv_to_dict(utf_csv_path)
        self.assertTrue("\\ud83c" in json.dumps(dict_value))



"""
pyxform is a Python library designed to make authoring XForms for ODK
Collect easy.
"""

__version__ = '0.10.0'

from pyxform.survey import Survey  # noqa
from pyxform.section import Section  # noqa
from pyxform.question import MultipleChoiceQuestion, InputQuestion, Question  # noqa
from pyxform.instance import SurveyInstance  # noqa
from pyxform.builder import SurveyElementBuilder, create_survey_from_xls, create_survey_element_from_dict, create_survey, create_survey_from_path  # noqa
from pyxform.question_type_dictionary import QUESTION_TYPE_DICT  # noqa
from pyxform.xls2json import SurveyReader as ExcelSurveyReader  # noqa

# This is what gets imported when someone imports pyxform



"""
Testing the instance object for pyxform.
"""
from unittest import TestCase
from pyxform import *
from pyxform.builder import create_survey_element_from_dict
from pyxform.tests.utils import prep_class_config


class Json2XformExportingPrepTests(TestCase):

    @classmethod
    def setUpClass(cls):
        prep_class_config(cls=cls)

    def test_simple_survey_instantiation(self):
        surv = Survey(name=u"Simple")
        q = create_survey_element_from_dict(
            {u"type": u"text",
             u"name": u"survey_question",
             u"label": u"Question"})
        surv.add_child(q)
        
        i = surv.instantiate()
        
        self.assertEquals(i.keys(), [u"survey_question"])
        self.assertEquals(set(i.xpaths()),
                          {u"/Simple", u"/Simple/survey_question"})
    
    def test_simple_survey_answering(self):
        surv = Survey(name=u"Water")
        q = create_survey_element_from_dict({
            u"type": u"text",
            u"name": u"color",
            u"label": u"Color"})
        q2 = create_survey_element_from_dict({
            u"type": u"text",
            u"name": u"feeling",
            u"label": u"Feeling"})
        
        surv.add_child(q)
        surv.add_child(q2)
        i = SurveyInstance(surv)
        
        i.answer(name=u"color", value=u"blue")
        self.assertEquals(i.answers()[u'color'], u"blue")
        
        i.answer(name=u"feeling", value=u"liquidy")
        self.assertEquals(i.answers()[u'feeling'], u"liquidy")
        
    def test_answers_can_be_imported_from_xml(self):
        surv = Survey(name=u"data")
        
        surv.add_child(create_survey_element_from_dict({
            u'type': u'text', u'name': u'name', u"label": u"Name"}))
        surv.add_child(create_survey_element_from_dict({
            u'type': u'integer', u'name': u'users_per_month',
            u"label": u"Users per month"}))
        surv.add_child(create_survey_element_from_dict({
            u'type': u'gps', u'name': u'geopoint', u'label': u'gps'}))
        surv.add_child(create_survey_element_from_dict({
            u'type': u'imei', u'name': u'device_id'}))
        
        instance = surv.instantiate()
        import_xml = self.config.get(
            self.cls_name, "test_answers_can_be_imported_from_xml")
        instance.import_from_xml(import_xml)
        
    def test_simple_registration_xml(self):
        reg_xform = Survey(name=u"Registration")
        name_question = create_survey_element_from_dict({
            u'type': u'text', u'name': u'name', u"label": u"Name"})
        reg_xform.add_child(name_question)
        
        reg_instance = reg_xform.instantiate()
        
        reg_instance.answer(name=u"name", value=u"bob")
        
        rx = reg_instance.to_xml()
        expected_xml = self.config.get(
            self.cls_name, "test_simple_registration_xml"
        ).format(reg_xform.id_string)
        self.assertEqual(rx, expected_xml)



from setuptools import setup, find_packages

setup(
    name='pyxform',
    version='0.10.0',
    author='github.com/xlsform',
    author_email='info@xlsform.org',
    packages=find_packages(),
    package_data={
        'pyxform.odk_validate': [
            'ODK_Validate.jar',
        ],
        'pyxform.tests': [
            'example_xls/*.*',
            'bug_example_xls/*.*',
            'test_output/*.*',
            'test_expected_output/*.*',
        ]
    },
    url='http://pypi.python.org/pypi/pyxform/',
    description='A Python package to create XForms for ODK Collect.',
    long_description=open('README.rst', 'rt').read(),
    install_requires=[
        'xlrd>=1.1.0',
        'unicodecsv>=0.14.1',
        'formencode',
        'unittest2',
    ],
    entry_points={
        'console_scripts': [
            'xls2xform=pyxform.xls2xform:main_cli',
        ],
    },
)



import copy
import os

from pyxform import file_utils, utils
from pyxform.errors import PyXFormError
from pyxform.question import (InputQuestion, MultipleChoiceQuestion,
                              OsmUploadQuestion, Question, RangeQuestion,
                              TriggerQuestion, UploadQuestion)
from pyxform.question_type_dictionary import QUESTION_TYPE_DICT
from pyxform.section import GroupedSection, RepeatingSection
from pyxform.survey import Survey
from pyxform.utils import unicode
from pyxform.xls2json import SurveyReader


def copy_json_dict(json_dict):
    """
    Returns a deep copy of the input json_dict
    """
    json_dict_copy = None
    items = None

    if type(json_dict) is list:
        json_dict_copy = [None]*len(json_dict)
        items = enumerate(json_dict)
    elif type(json_dict) is dict:
        json_dict_copy = {}
        items = json_dict.items()

    for key, value in items:
        if type(value) is dict or type(value) is list:
            json_dict_copy[key] = copy_json_dict(value)
        else:
            json_dict_copy[key] = value

    return json_dict_copy


class SurveyElementBuilder(object):
    # we use this CLASSES dict to create questions from dictionaries
    QUESTION_CLASSES = {
        u"": Question,
        u"input": InputQuestion,
        u"trigger": TriggerQuestion,
        u"select": MultipleChoiceQuestion,
        u"select1": MultipleChoiceQuestion,
        u"upload": UploadQuestion,
        u"osm": OsmUploadQuestion,
        u'range': RangeQuestion,
        }

    SECTION_CLASSES = {
        u"group": GroupedSection,
        u"repeat": RepeatingSection,
        u"survey": Survey,
        }

    def __init__(self, **kwargs):
        # I don't know why we would need an explicit none option for
        # select alls
        self._add_none_option = False
        self.set_sections(
            kwargs.get(u"sections", {})
            )

    def set_sections(self, sections):
        """
        sections is a dict of python objects, a key in this dict is
        the name of the section and the value is a dict that can be
        used to create a whole survey.
        """
        assert type(sections) == dict
        self._sections = sections

    def create_survey_element_from_dict(self, d):
        """
        Convert from a nested python dictionary/array structure (a json dict I
        call it because it corresponds directly with a json object)
        to a survey object
        """
        if u"add_none_option" in d:
            self._add_none_option = d[u"add_none_option"]
        if d[u"type"] in self.SECTION_CLASSES:
            return self._create_section_from_dict(d)
        elif d[u"type"] == u"loop":
            return self._create_loop_from_dict(d)
        elif d[u"type"] == u"include":
            section_name = d[u"name"]
            if section_name not in self._sections:
                raise PyXFormError("This section has not been included.",
                                   section_name, self._sections.keys())
            d = self._sections[section_name]
            full_survey = self.create_survey_element_from_dict(d)
            return full_survey.children
        else:
            return self._create_question_from_dict(
                d, copy_json_dict(QUESTION_TYPE_DICT), self._add_none_option)

    @staticmethod
    def _create_question_from_dict(d, question_type_dictionary,
                                   add_none_option=False):
        question_type_str = d[u"type"]
        d_copy = d.copy()

        # TODO: Keep add none option?
        if add_none_option \
                and question_type_str.startswith(u"select all that apply"):
            SurveyElementBuilder\
                ._add_none_option_to_select_all_that_apply(d_copy)

        # Handle or_other on select type questions
        or_other_str = u" or specify other"
        if question_type_str.endswith(or_other_str):
            question_type_str = \
                question_type_str[:len(question_type_str) - len(or_other_str)]
            d_copy["type"] = question_type_str
            SurveyElementBuilder\
                ._add_other_option_to_multiple_choice_question(d_copy)
            return [
                SurveyElementBuilder._create_question_from_dict(
                    d_copy, question_type_dictionary, add_none_option),
                SurveyElementBuilder._create_specify_other_question_from_dict(
                    d_copy)]

        question_class = SurveyElementBuilder._get_question_class(
            question_type_str, question_type_dictionary)

        # todo: clean up this spaghetti code
        d_copy[u"question_type_dictionary"] = question_type_dictionary
        if question_class:

            return question_class(**d_copy)

        return []

    @staticmethod
    def _add_other_option_to_multiple_choice_question(d):
        # ideally, we'd just be pulling from children
        choice_list = d.get(u"choices", d.get(u"children", []))
        if len(choice_list) <= 0:
            raise PyXFormError("There should be choices for this question.")
        other_choice = {
            u"name": u"other",
            u"label": u"Other",
            }
        if other_choice not in choice_list:
            choice_list.append(other_choice)

    @staticmethod
    def _add_none_option_to_select_all_that_apply(d_copy):
        choice_list = d_copy.get(u"choices", d_copy.get(u"children", []))
        if len(choice_list) <= 0:
            raise PyXFormError("There should be choices for this question.")
        none_choice = {
            u"name": u"none",
            u"label": u"None",
            }
        if none_choice not in choice_list:
            choice_list.append(none_choice)
            none_constraint = u"(.='none' or not(selected(., 'none')))"
            if u"bind" not in d_copy:
                d_copy[u"bind"] = {}
            if u"constraint" in d_copy[u"bind"]:
                d_copy[u"bind"][u"constraint"] += " and " + none_constraint
            else:
                d_copy[u"bind"][u"constraint"] = none_constraint

    @staticmethod
    def _get_question_class(question_type_str, question_type_dictionary):
        """
        Read the type string from the json format,
        and find what class it maps to going through
        type_dictionary -> QUESTION_CLASSES
        """
        question_type = question_type_dictionary.get(question_type_str, {})
        control_dict = question_type.get(u"control", {})
        control_tag = control_dict.get(u"tag", u"")
        if control_tag == u"upload" \
                and control_dict.get(u"mediatype") == "osm/*":
            control_tag = u"osm"

        return SurveyElementBuilder.QUESTION_CLASSES[control_tag]

    @staticmethod
    def _create_specify_other_question_from_dict(d):
        kwargs = {
            u"type": u"text",
            u"name": u"%s_other" % d[u"name"],
            u"label": u"Specify other.",
            u"bind": {u"relevant": u"selected(../%s, 'other')" % d[u"name"]},
            }
        return InputQuestion(**kwargs)

    def _create_section_from_dict(self, d):
        d_copy = d.copy()
        children = d_copy.pop(u"children", [])
        section_class = self.SECTION_CLASSES[d_copy[u"type"]]
        if d[u'type'] == u'survey' and u'title' not in d:
            d_copy[u'title'] = d[u'name']
        result = section_class(**d_copy)
        for child in children:
            # Deep copying the child is a hacky solution to the or_other bug.
            # I don't know why it works.
            # And I hope it doesn't break something else.
            # I think the good solution would be to rewrite this class.
            survey_element = self.create_survey_element_from_dict(
                copy.deepcopy(child))
            if survey_element:
                result.add_children(survey_element)

        return result

    def _create_loop_from_dict(self, d):
        """
        Takes a json_dict of "loop" type
        Returns a GroupedSection
        """
        d_copy = d.copy()
        children = d_copy.pop(u"children", [])
        columns = d_copy.pop(u"columns", [])
        result = GroupedSection(**d_copy)

        # columns is a left over from when this was
        # create_table_from_dict, I will need to clean this up
        for column_dict in columns:
            # If this is a none option for a select all that apply
            # question then we should skip adding it to the result
            if column_dict[u"name"] == "none":
                continue

            column = GroupedSection(**column_dict)
            for child in children:
                question_dict = self._name_and_label_substitutions(
                    child, column_dict)
                question = self.create_survey_element_from_dict(question_dict)
                column.add_child(question)
            result.add_child(column)
        if result.name != u"":
            return result

        # TODO: Verify that nothing breaks if this returns a list
        return result.children

    def _name_and_label_substitutions(self, question_template, column_headers):
        # if the label in column_headers has multiple languages setup a
        # dictionary by language to do substitutions.
        info_by_lang = {}
        if type(column_headers[u"label"]) == dict:
            info_by_lang = dict(
                [(lang, {u"name": column_headers[u"name"],
                         u"label": column_headers[u"label"][lang]})
                 for lang in column_headers[u"label"].keys()]
                )

        result = question_template.copy()
        for key in result.keys():
            if type(result[key]) == unicode:
                result[key] %= column_headers
            elif type(result[key]) == dict:
                result[key] = result[key].copy()
                for key2 in result[key].keys():
                    if type(column_headers[u"label"]) == dict:
                        result[key][key2] %= info_by_lang.get(
                            key2, column_headers)
                    else:
                        result[key][key2] %= column_headers
        return result

    def create_survey_element_from_json(self, str_or_path):
        d = utils.get_pyobj_from_json(str_or_path)
        return self.create_survey_element_from_dict(d)


def create_survey_element_from_dict(d, sections=None):
    """
    Creates a Survey from a dictionary in the format provided by SurveyReader
    """
    if sections is None:
        sections = {}
    builder = SurveyElementBuilder()
    builder.set_sections(sections)
    return builder.create_survey_element_from_dict(d)


def create_survey_element_from_json(str_or_path):
    d = utils.get_pyobj_from_json(str_or_path)
    return create_survey_element_from_dict(d)


def create_survey_from_xls(path_or_file):
    excel_reader = SurveyReader(path_or_file)
    d = excel_reader.to_json_dict()
    survey = create_survey_element_from_dict(d)
    if not survey.id_string:
        survey.id_string = excel_reader._name
    return survey


def create_survey(
        name_of_main_section=None, sections=None,
        main_section=None,
        id_string=None,
        title=None,
        default_language=None,
        ):
    """
    name_of_main_section -- a string key used to find the main section in the
                            sections dict if it is not supplied in the
                            main_section arg
    main_section -- a json dict that represents a survey
    sections -- a dictionary of sections that can be drawn from to build the
                survey
    This function uses the builder class to create and return a survey.
    """
    if sections is None:
        sections = {}
    if main_section is None:
        main_section = sections[name_of_main_section]
    builder = SurveyElementBuilder()
    builder.set_sections(sections)

    # assert name_of_main_section in sections, name_of_main_section
    if u"id_string" not in main_section:
        main_section[u"id_string"] = name_of_main_section \
            if id_string is None else name_of_main_section
    survey = builder.create_survey_element_from_dict(main_section)

    # not sure where to do this without repeating ourselves,
    # but it's needed to pass xls2xform tests
    # TODO: I would assume that the json-dict is valid
    # (i.e. that it has a id string), then throw an error here.
    #        We can set the id to whatever we want in xls2json.
    #        Although to be totally modular, maybe we do need to repeat a lot
    #       of the validation and setting default value stuff from xls2json
    if id_string is not None:
        survey.id_string = id_string

    if title is not None:
        survey.title = title
    survey.def_lang = default_language

    return survey


def create_survey_from_path(path, include_directory=False):
    """
    include_directory -- Switch to indicate that all the survey forms in the
                         same directory as the specified file should be read
                         so they can be included through include types.
    @see: create_survey
    """
    directory, file_name = os.path.split(path)
    if include_directory:
        main_section_name = file_utils._section_name(file_name)
        sections = file_utils.collect_compatible_files_in_directory(directory)
    else:
        main_section_name, section = file_utils.load_file_to_dict(path)
        sections = {main_section_name: section}
    pkg = {
        u'name_of_main_section': main_section_name,
        u'sections': sections
        }

    return create_survey(**pkg)



"""
Testing our ability to import from a JSON text file.
"""
from unittest import TestCase

from pyxform.builder import create_survey_element_from_dict


class Json2XformTestJsonImport(TestCase):
    def test_simple_questions_can_be_imported_from_json(self):
        json_text = {
            "type": "survey",
            "name": "Exchange rate",
            "children": [
                {
                    "label": {"French": "Combien?", "English": "How many?"},
                    "type": "decimal",
                    "name": "exchange_rate"
                }
            ]
        }
        s = create_survey_element_from_dict(json_text)

        self.assertEqual(s.children[0].type, "decimal")



"""
XLS-to-dict and csv-to-dict are essentially backends for xls2json.
"""
import xlrd
from xlrd import XLRDError
import unicodecsv as csv
from io import BytesIO
from pyxform import constants
import re
import datetime
from pyxform.errors import PyXFormError
from pyxform.utils import unicode, basestring, unichr
from functools import reduce
from collections import OrderedDict


def _list_to_dict_list(list_items):
    """
    Takes a list and creates a dict with the list values as keys.
    Returns a list of the created dict or an empty list
    """
    if list_items:
        k = OrderedDict()
        for item in list_items:
            k[u'%s' % item] = u''
        return [k]
    return []


def xls_to_dict(path_or_file):
    """
    Return a Python dictionary with a key for each worksheet
    name. For each sheet there is a list of dictionaries, each
    dictionary corresponds to a single row in the worksheet. A
    dictionary has keys taken from the column headers and values
    equal to the cell value for that row and column.
    All the keys and leaf elements are unicode text.
    """
    try:
        if isinstance(path_or_file, basestring):
            workbook = xlrd.open_workbook(filename=path_or_file)
        else:
            workbook = xlrd.open_workbook(file_contents=path_or_file.read())
    except XLRDError as e:
        raise PyXFormError("Error reading .xls file: %s" % e.message)

    def xls_value_to_unicode(value, value_type):
        """
        Take a xls formatted value and try to make a unicode string
        representation.
        """
        if value_type == xlrd.XL_CELL_BOOLEAN:
            return u"TRUE" if value else u"FALSE"
        elif value_type == xlrd.XL_CELL_NUMBER:
            # Try to display as an int if possible.
            int_value = int(value)
            if int_value == value:
                return unicode(int_value)
            else:
                return unicode(value)
        elif value_type is xlrd.XL_CELL_DATE:
            # Warn that it is better to single quote as a string.
            # error_location = cellFormatString % (ss_row_idx, ss_col_idx)
            # raise Exception(
            #   "Cannot handle excel formatted date at " + error_location)
            datetime_or_time_only = xlrd.xldate_as_tuple(
                value, workbook.datemode)
            if datetime_or_time_only[:3] == (0, 0, 0):
                # must be time only
                return unicode(datetime.time(*datetime_or_time_only[3:]))
            return unicode(datetime.datetime(*datetime_or_time_only))
        else:
            # ensure unicode and replace nbsp spaces with normal ones
            # to avoid this issue:
            # https://github.com/modilabs/pyxform/issues/83
            return unicode(value).replace(unichr(160), ' ')

    def xls_to_dict_normal_sheet(sheet):
        def iswhitespace(string):
            return (
                isinstance(string, basestring) and len(string.strip()) == 0)

        # Check for duplicate column headers
        column_header_list = list()
        for column in range(0, sheet.ncols):
            column_header = sheet.cell_value(0, column)
            if column_header in column_header_list:
                raise PyXFormError(
                    u"Duplicate column header: %s" % column_header)
            # xls file with 3 columns mostly have a 3 more columns that are
            # blank by default or something, skip during check
            if column_header is not None:
                if not iswhitespace(column_header):
                    column_header_list.append(column_header)

        result = []
        for row in range(1, sheet.nrows):
            row_dict = OrderedDict()
            for column in range(0, sheet.ncols):
                # Changing to cell_value function
                # convert to string, in case it is not string
                key = u"%s" % sheet.cell_value(0, column)
                key = key.strip()
                value = sheet.cell_value(row, column)
                # remove whitespace at the beginning and end of value
                if isinstance(value, basestring):
                    value = value.strip()
                value_type = sheet.cell_type(row, column)
                if value is not None:
                    if not iswhitespace(value):
                        row_dict[key] = xls_value_to_unicode(value, value_type)
                # Taking this condition out so I can get accurate row numbers.
                # TODO: Do the same for csvs
                # if row_dict != {}:
            result.append(row_dict)
        return result, _list_to_dict_list(column_header_list)

    def xls_value_from_sheet(sheet, row, column):
        value = sheet.cell_value(row, column)
        value_type = sheet.cell_type(row, column)
        if value is not None and value != "":
            return xls_value_to_unicode(value, value_type)
        else:
            raise PyXFormError("Empty Value")

    def _xls_to_dict_cascade_sheet(sheet):
        result = []
        rs_dict = OrderedDict()  # tmp dict to hold entire structure

        def slugify(s):
            return re.sub(r'\W+', '_', s.strip().lower())

        prefix = "$PREFIX$"
        # get col headers and position first, ignore first column
        for column in range(1, sheet.ncols):
            col_name = sheet.cell_value(0, column)
            rs_dict[col_name] = {
                'pos': column,
                'data': [],
                'itemset': col_name,
                'type': constants.SELECT_ONE,
                'name':
                    prefix if (column == sheet.ncols - 1) else u''.join(
                        [prefix, '_', col_name]),
                'label': sheet.cell_value(1, column)}
            if column > 1:
                rs_dict[col_name]['parent'] = sheet.cell_value(0, column - 1)
            else:
                rs_dict[col_name]['choices'] = []
            choice_filter = ''
            for a in range(1, column):
                prev_col_name = sheet.cell_value(0, a)
                if choice_filter != '':
                    choice_filter += ' and %s=${%s_%s}' % \
                                     (prev_col_name, prefix, prev_col_name)
                else:
                    choice_filter += '%s=${%s_%s}' % \
                                     (prev_col_name, prefix, prev_col_name)
            rs_dict[col_name]['choice_filter'] = choice_filter
        # get data, use new cascade dict structure, data starts on 3 row
        for row in range(2, sheet.nrows):
            # go through each header aka column
            for col_name in rs_dict:
                column = rs_dict[col_name]['pos']
                cell_data = xls_value_from_sheet(sheet, row, column)
                try:
                    rs_dict[col_name]['data'].index(slugify(cell_data))
                except ValueError:
                    rs_dict[col_name]['data'].append(slugify(cell_data))
                    if 'choices' in rs_dict[col_name]:
                        l = {'name': slugify(cell_data), 'label': cell_data}
                        rs_dict[col_name]['choices'].append(l)
                data = {
                    'name': slugify(cell_data),
                    'label': cell_data.strip(),
                    constants.LIST_NAME: col_name
                }
                for prev_column in range(1, column):
                    prev_col_name = sheet.cell_value(0, prev_column)
                    data[prev_col_name] = slugify(xls_value_from_sheet(
                        sheet, row, prev_column))
                result.append(data)
        # order
        kl = []
        for column in range(1, sheet.ncols):
            col_name = sheet.cell_value(0, column)
            if 'parent' in rs_dict[col_name]:
                rs_dict[col_name].pop('parent')
            if 'pos' in rs_dict[col_name]:
                rs_dict[col_name].pop('pos')
            if 'data' in rs_dict[col_name]:
                rs_dict[col_name].pop('data')
            kl.append(rs_dict[col_name])

            # create list with no duplicates
        choices = []
        for rec in result:
            c = 0
            for check in result:
                if rec == check:
                    c += 1
            if c == 1:
                choices.append(rec)
            else:
                try:
                    choices.index(rec)
                except ValueError:
                    choices.append(rec)
        return [{'choices': choices, 'questions': kl}]

    result = OrderedDict()
    for sheet in workbook.sheets():
        if sheet.name == constants.CASCADING_CHOICES:
            result[sheet.name] = _xls_to_dict_cascade_sheet(sheet)
        else:
            result[sheet.name], result[u"%s_header" % sheet.name] = \
                xls_to_dict_normal_sheet(sheet)
    return result


def get_cascading_json(sheet_list, prefix, level):
    return_list = []
    for row in sheet_list:
        if 'stopper' in row:
            if row['stopper'] == level:
                # last element's name IS the prefix; doesn't need level
                return_list[-1]["name"] = prefix
                return return_list
            else:
                continue
        elif 'lambda' in row:
            def replace_prefix(d, prefix):
                for k, v in d.items():
                    if isinstance(v, basestring):
                        d[k] = v.replace('$PREFIX$', prefix)
                    elif isinstance(v, dict):
                        d[k] = replace_prefix(v, prefix)
                    elif isinstance(v, list):
                        d[k] = map(lambda x: replace_prefix(x, prefix), v)
                return d

            return_list.append(replace_prefix(row['lambda'], prefix))
    raise PyXFormError("Found a cascading_select " + level +
                       ", but could not find " + level + "in cascades sheet.")


def csv_to_dict(path_or_file):
    if isinstance(path_or_file, basestring):
        csv_data = open(path_or_file, 'rb')
    else:
        csv_data = path_or_file

    _dict = OrderedDict()

    def first_column_as_sheet_name(row):
        if len(row) == 0:
            return None, None
        elif len(row) == 1:
            return row[0], None
        else:
            s_or_c = row[0]
            content = row[1:]
            if s_or_c == '':
                s_or_c = None
            # concatenate all the strings in content
            if reduce(lambda x, y: x + y, content) == '':
                # content is a list of empty strings
                content = None
            return s_or_c, content

    reader = csv.reader(csv_data, encoding='utf-8')
    sheet_name = None
    current_headers = None
    for row in reader:
        survey_or_choices, content = first_column_as_sheet_name(row)
        if survey_or_choices is not None:
            sheet_name = survey_or_choices
            if sheet_name not in _dict:
                _dict[unicode(sheet_name)] = []
            current_headers = None
        if content is not None:
            if current_headers is None:
                current_headers = content
                _dict[u"%s_header" % sheet_name] = \
                    _list_to_dict_list(current_headers)
            else:
                _d = OrderedDict()
                for key, val in zip(current_headers, content):
                    if val != "":
                        # Slight modification so values are striped
                        # this is because csvs often spaces following commas
                        # (but the csv reader might already handle that.)
                        _d[unicode(key)] = unicode(val.strip())
                _dict[sheet_name].append(_d)
    csv_data.close()
    return _dict


"""
I want the ability to go:

    xls => csv
so that we can go:
    xls => csv => survey

and some day:
    csv => xls

"""


def convert_file_to_csv_string(path):
    """
    This will open a csv or xls file and return a CSV in the format:
        sheet_name1
        ,col1,col2
        ,r1c1,r1c2
        ,r2c1,r2c2
        sheet_name2
        ,col1,col2
        ,r1c1,r1c2
        ,r2c1,r2c2

    Currently, it processes csv files and xls files to ensure consistent
    csv delimiters, etc. for tests.
    """
    if path.endswith(".csv"):
        imported_sheets = csv_to_dict(path)
    else:
        imported_sheets = xls_to_dict(path)
    foo = BytesIO()
    writer = csv.writer(
        foo, delimiter=',', quotechar='"', quoting=csv.QUOTE_MINIMAL)
    for sheet_name, rows in imported_sheets.items():
        writer.writerow([sheet_name])
        out_keys = []
        out_rows = []
        for row in rows:
            out_row = []
            for key in row.keys():
                if key not in out_keys:
                    out_keys.append(key)
            for out_key in out_keys:
                out_row.append(row.get(out_key, None))
            out_rows.append(out_row)
        writer.writerow([None] + out_keys)
        for out_row in out_rows:
            writer.writerow([None] + out_row)
    return foo.getvalue().decode('utf-8')



from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class XlsFormHeadersTest(PyxformTestCase):
    def test_label_caps_alternatives(self):
        """
        re: https://github.com/SEL-Columbia/pyxform/issues/76
        Capitalization of 'label' column can lead to confusing errors.
        """
        s1 = self.md_to_pyxform_survey("""
            | survey |      |      |       |
            |        | type | name | label |
            |        | note | q    | Q     |
            """)
        s2 = self.md_to_pyxform_survey("""
            | survey |      |      |       |
            |        | type | name | Label | # <-- note: capital L
            |        | note | q    | Q     |
            """)
        self.assertEqual(s1.to_xml(), s2.to_xml())

    def test_calculate_alias(self):
        self.assertPyxformXform(
            name="calculatealias",
            md="""
            | survey |           |         |         |               |
            |        | type      | name    | label   | calculate     |
            |        | decimal   | amount  | Counter |               |
            |        | calculate | doubled | Doubled | ${amount} * 2 |
            """,
            errored=False,
            debug=True)



"""
A Python script to convert excel files into JSON.
"""
from __future__ import print_function, unicode_literals
import json
import re
import sys
import codecs
import os
from pyxform import constants, aliases
from pyxform.errors import PyXFormError
from pyxform.xls2json_backends import xls_to_dict, csv_to_dict
from pyxform.utils import is_valid_xml_tag, unicode, basestring

SMART_QUOTES = {
    '\u2018': "'",
    '\u2019': "'",
    '\u201c': '"',
    '\u201d': '"',
}


def print_pyobj_to_json(pyobj, path=None):
    """
    dump a python nested array/dict structure to the specified file
    or stdout if no file is specified
    """
    if path:
        fp = codecs.open(path, mode="w", encoding="utf-8")
        json.dump(pyobj, fp=fp, ensure_ascii=False, indent=4)
        fp.close()
    else:
        print(json.dumps(pyobj, ensure_ascii=False, indent=4))


def merge_dicts(dict_a, dict_b, default_key="default"):
    """
    Recursively merge two nested dicts into a single dict.
    When keys match their values are merged using
    a recursive call to this function,
    otherwise they are just added to the output dict.
    """
    if dict_a is None or dict_a == {}:
        return dict_b
    if dict_b is None or dict_b == {}:
        return dict_a

    if type(dict_a) is not dict:
        if default_key in dict_b:
            return dict_b
        dict_a = {default_key: dict_a}
    if type(dict_b) is not dict:
        if default_key in dict_a:
            return dict_a
        dict_b = {default_key: dict_b}

    all_keys = set(dict_a.keys()).union(set(dict_b.keys()))

    out_dict = dict()
    for key in all_keys:
        out_dict[key] = merge_dicts(
            dict_a.get(key), dict_b.get(key), default_key)
    return out_dict


def list_to_nested_dict(lst):
    """
    [1,2,3,4] -> {1:{2:{3:4}}}
    """
    if len(lst) > 1:
        return {lst[0]: list_to_nested_dict(lst[1:])}
    else:
        return lst[0]


def replace_smart_quotes_in_dict(_d):
    for key, value in _d.items():
        _changed = False
        for smart_quote, dumb_quote in SMART_QUOTES.items():
            if smart_quote in value:
                value = value.replace(smart_quote, dumb_quote)
                _changed = True
        if _changed:
            _d[key] = value


def dealias_and_group_headers(dict_array, header_aliases, use_double_colons,
                              default_language=u"default", ignore_case=False):
    """
    For each row in the worksheet, group all keys that contain a double colon.
    So
        {"text::english": "hello", "text::french" : "bonjour"}
    becomes
        {"text": {"english": "hello", "french" : "bonjour"}.
    Dealiasing is done to the first token
    (the first term separated by the delimiter).
    default_language -- used to group labels/hints/etc
    without a language specified with localized versions.
    """
    group_delimiter = u"::"
    out_dict_array = list()
    for row in dict_array:
        out_row = dict()
        for header, val in row.items():

            if ignore_case:
                header = header.lower()

            if use_double_colons:
                tokens = header.split(group_delimiter)

            # else:
            #   We do the initial parse using single colons
            #   for backwards compatibility and
            #   only the first single is used
            #   in order to avoid nesting jr:something tokens.
            #   if len(tokens) > 1:
            #       tokens[1:] = [u":".join(tokens[1:])]
            else:
                # I think the commented out section above
                # break if there is something like media:image:english
                # so maybe a better backwards compatibility hack
                # is to join any jr token with the next token
                tokens = header.split(u":")
                if "jr" in tokens:
                    jr_idx = tokens.index("jr")
                    tokens[jr_idx] = u":".join(tokens[jr_idx: jr_idx + 2])
                    tokens.pop(jr_idx + 1)

            dealiased_first_token = header_aliases.get(tokens[0], tokens[0])
            tokens = dealiased_first_token.split(group_delimiter) + tokens[1:]
            new_key = tokens[0]
            new_value = list_to_nested_dict(tokens[1:] + [val])
            out_row = merge_dicts(
                out_row, {new_key: new_value}, default_language)

        out_dict_array.append(out_row)
    return out_dict_array


def dealias_types(dict_array):
    """
    Look at all the type values in a dict array and if any aliases are found,
    replace them with the name they map to.
    """
    for row in dict_array:
        found_type = row.get(constants.TYPE)
        if found_type in aliases._type.keys():
            row[constants.TYPE] = aliases._type[found_type]
    return dict_array


def clean_text_values(dict_array):
    """
    Go though the dict array and strips all text values.
    Also replaces multiple spaces with single spaces.
    Note that the keys don't get cleaned, which could be an issue.
    """
    for row in dict_array:
        replace_smart_quotes_in_dict(row)
        for key, value in row.items():
            if isinstance(value, basestring):
                row[key] = re.sub(r"( )+", " ", value.strip())
    return dict_array


# This is currently unused because name uniqueness is checked in json2xform.
def check_name_uniqueness(dict_array):
    """
    Make sure all names are unique
    Raises and exception if a duplicate is found
    """
    # This set is used to validate the uniqueness of names.
    name_set = set()
    row_number = 0  # TODO: There might be a bug with row numbers...
    for row in dict_array:
        row_number += 1
        name = row.get(constants.NAME)
        if name:
            if name in name_set:
                raise PyXFormError(
                    "Question name is not unique: " +
                    str(name) + " Row: " + str(row_number))
            else:
                name_set.add(name)
    return dict_array


def group_dictionaries_by_key(list_of_dicts, key, remove_key=True):
    """
    Takes a list of dictionaries and returns a
    dictionary of lists of dictionaries with the same value for the given key.
    The grouping key is removed by default.
    If the key is not in any dictionary an empty dict is returned.
    """
    dict_of_lists = dict()
    for dicty in list_of_dicts:
        if key not in dicty:
            continue
        dicty_value = dicty[key]
        if remove_key:
            dicty.pop(key)
        if dicty_value in dict_of_lists:
            dict_of_lists[dicty_value].append(dicty)
        else:
            dict_of_lists[dicty_value] = [dicty]
    return dict_of_lists


def has_double_colon(workbook_dict):
    """
    Look for a column header with a doublecolon (::) and
    return true if one is found.
    """
    for sheet in workbook_dict.values():
        for row in sheet:
            for column_header in row.keys():
                if type(column_header) is not unicode:
                    continue
                if u"::" in column_header:
                    return True
    return False


def add_flat_annotations(prompt_list, parent_relevant='', name_prefix=''):
    """
    This is a helper function for generating flat instances
    for the benefit of ODK Tables.
    It makes the following modifications to the survey:
    X Renames prompts with their group name as a prefix
      (Decided against chaning the names because it breaks formulas)
      (However, there could be namespace collisions now.)
    - "and"s group relevance formulas onto that of their children.
    - Adds a flat property to groups
      The flat property is used in the json2xform code
    """
    for prompt in prompt_list:
        prompt_relevant = prompt.get('bind', {}).get('relevant', '')
        new_relevant = ''
        if parent_relevant != '':
            new_relevant += parent_relevant
            if prompt_relevant != '':
                new_relevant += ' and (' + prompt_relevant + ')'
        elif prompt_relevant != '':
            new_relevant = prompt_relevant

        children = prompt.get(constants.CHILDREN)
        if children:
            prompt['flat'] = True
            add_flat_annotations(children, new_relevant,
                                 name_prefix + '_' + prompt['name'])
        else:
            if new_relevant != '':
                prompt['bind'] = prompt.get('bind', {})
                prompt['bind']['relevant'] = new_relevant
                # if name_prefix != '':
                #    prompt['name'] = name_prefix + prompt['name']


def process_range_question_type(row):
    """Returns a new row that includes the Range parameters start, end and
    step.

    Raises PyXFormError when invalid range parameters are used.
    """
    def _parameters(parameters):
        parts = parameters.split(';')
        if len(parts) == 1:
            parts = parameters.split(',')
        if len(parts) == 1:
            parts = parameters.split()

        return parts

    new_dict = row.copy()
    parameters = _parameters(new_dict.get('parameters', ''))
    parameters_map = {'start': 'start', 'end': 'end', 'step': 'step'}
    defaults = {'start': '1', 'end': '10', 'step': '1'}
    params = {}

    for param in parameters:
        if '=' not in param:
            raise PyXFormError("Expecting parameters to be in the form of "
                               "'start=X end=X step=X'.")
        k, v = param.split('=')[:2]
        key = parameters_map.get(k.lower().strip())
        if key:
            params[key] = v.strip()
        else:
            raise PyXFormError(
                "Range has the parameters 'start', 'end' and"
                " 'step': '%s' is an invalid parameter." % k)

    # set defaults
    for key in parameters_map.values():
        if key not in params:
            params[key] = defaults[key]

    try:
        has_float = any(
            [float(x) and '.' in str(x) for x in params.values()])
    except ValueError:
        raise PyXFormError("Range parameters 'start', "
                           "'end' or 'step' must all be numbers.")
    else:
        # is integer by default, convert to decimal if it has any float values
        if has_float:
            new_dict['bind'] = new_dict.get('bind', {})
            new_dict['bind'].update({'type': 'decimal'})

    new_dict['parameters'] = params

    return new_dict


def workbook_to_json(
        workbook_dict, form_name=None,
        default_language=u"default", warnings=None):
    """
    workbook_dict -- nested dictionaries representing a spreadsheet.
                    should be similar to those returned by xls_to_dict
    form_name -- The spreadsheet's filename
    default_language -- default_language does two things:
    1. In the xform the default language is the language reverted to when
       there is no translation available for some itext element. Because
       of this every itext element must have a default language translation.
    2. In the workbook if media/labels/hints that do not have a
       language suffix will be treated as though their suffix is the
       default language.
       If the default language is used as a suffix for media/labels/hints,
       then the suffixless version will be overwritten.
    warnings -- an optional list which warnings will be appended to

    returns a nested dictionary equivalent to the format specified in the
    json form spec.
    """
    # ensure required headers are present
    if warnings is None:
        warnings = []
    is_valid = False
    workbook_dict = {x.lower(): y for x,y in workbook_dict.items()}
    for row in workbook_dict.get(constants.SURVEY, []):
        is_valid = 'type' in [z.lower() for z in row]
        if is_valid:
            break
    if not is_valid:
        raise PyXFormError(
            u"The survey sheet is either empty or missing important "
            u"column headers.")

    row_format_string = '[row : %s]'

    # Make sure the passed in vars are unicode
    form_name = unicode(form_name)
    default_language = unicode(default_language)

    # We check for double columns to determine whether to use them
    # or single colons to delimit grouped headers.
    # Single colons are bad because they conflict with with the xform namespace
    # syntax (i.e. jr:constraintMsg),
    # so we only use them if we have to for backwards compatibility.
    use_double_colons = has_double_colon(workbook_dict)

    # Break the spreadsheet dict into easier to access objects
    # (settings, choices, survey_sheet):
    # ########## Settings sheet ##########
    settings_sheet = dealias_and_group_headers(
        workbook_dict.get(constants.SETTINGS, []),
        aliases.settings_header, use_double_colons)
    settings = settings_sheet[0] if len(settings_sheet) > 0 else {}
    replace_smart_quotes_in_dict(settings)

    default_language = settings.get(
        constants.DEFAULT_LANGUAGE, default_language)

    # add_none_option is a boolean that when true,
    # indicates a none option should automatically be added to selects.
    # It should probably be deprecated but I haven't checked yet.
    if u"add_none_option" in settings:
        settings[u"add_none_option"] = aliases.yes_no.get(
            settings[u"add_none_option"], False)

    # Here we create our json dict root with default settings:
    id_string = settings.get(constants.ID_STRING, form_name)
    sms_keyword = settings.get(constants.SMS_KEYWORD, id_string)
    json_dict = {
        constants.TYPE: constants.SURVEY,
        constants.NAME: form_name,
        constants.TITLE: id_string,
        constants.ID_STRING: id_string,
        constants.SMS_KEYWORD: sms_keyword,
        constants.DEFAULT_LANGUAGE: default_language,
        # By default the version is based on the date and time yyyymmddhh
        # Leaving default version out for now since it might cause
        # problems for formhub.
        # constants.VERSION : datetime.datetime.now().strftime("%Y%m%d%H"),
        constants.CHILDREN: []
    }
    # Here the default settings are overridden by those in the settings sheet
    json_dict.update(settings)

    # ########## Choices sheet ##########
    # Columns and "choices and columns" sheets are deprecated,
    # but we combine them with the choices sheet for backwards-compatibility.
    choices_and_columns_sheet = workbook_dict.get(
        constants.CHOICES_AND_COLUMNS, {})
    choices_and_columns_sheet = dealias_and_group_headers(
        choices_and_columns_sheet, aliases.list_header,
        use_double_colons, default_language)

    columns_sheet = workbook_dict.get(constants.COLUMNS, [])
    columns_sheet = dealias_and_group_headers(
        columns_sheet, aliases.list_header,
        use_double_colons, default_language)

    choices_sheet = workbook_dict.get(constants.CHOICES, [])
    for choice_item in choices_sheet:
        replace_smart_quotes_in_dict(choice_item)

    choices_sheet = dealias_and_group_headers(
        choices_sheet, aliases.list_header, use_double_colons,
        default_language)
    # ########## Cascading Select sheet ###########
    cascading_choices = workbook_dict.get(constants.CASCADING_CHOICES, [])
    if len(cascading_choices):
        if 'choices' in cascading_choices[0]:
            choices_sheet = choices_sheet + cascading_choices[0]['choices']

    combined_lists = group_dictionaries_by_key(
        choices_and_columns_sheet + choices_sheet + columns_sheet,
        constants.LIST_NAME)

    choices = combined_lists
    # Make sure all the options have the required properties:
    warnedabout = set()
    for list_name, options in choices.items():
        for option in options:
            if 'name' not in option:
                info = "[list_name : " + list_name + ']'
                raise PyXFormError("On the choices sheet there is "
                                   "a option with no name. " + info)
            if 'label' not in option:
                info = "[list_name : " + list_name + ']'
                warnings.append(
                    "On the choices sheet there is a option with no label. " +
                    info)
            # chrislrobert's fix for a cryptic error message:
            # see: https://code.google.com/p/opendatakit/issues/detail?id=832&start=200 # noqa
            option_keys = list(option.keys())
            for headername in option_keys:
                # Using warnings and removing the bad columns
                # instead of throwing errors because some forms
                # use choices column headers for notes.
                if ' ' in headername:
                    if headername not in warnedabout:
                        warnedabout.add(headername)
                        warnings.append("On the choices sheet there is " +
                                        "a column (\"" +
                                        headername +
                                        "\") with an illegal header. " +
                                        "Headers cannot include spaces.")
                    del option[headername]
                elif headername == '':
                    warnings.append("On the choices sheet there is a value" +
                                    " in a column with no header.")
                    del option[headername]
    # ########## Survey sheet ###########
    if constants.SURVEY not in workbook_dict:
        raise PyXFormError(
            "You must have a sheet named (case-sensitive): " +
            constants.SURVEY)
    survey_sheet = workbook_dict[constants.SURVEY]
    # Process the headers:
    clean_text_values_enabled = aliases.yes_no.get(
        settings.get("clean_text_values", "true()"))
    if clean_text_values_enabled:
        survey_sheet = clean_text_values(survey_sheet)
    survey_sheet = dealias_and_group_headers(
        survey_sheet, aliases.survey_header,
        use_double_colons, default_language)
    survey_sheet = dealias_types(survey_sheet)

    osm_sheet = dealias_and_group_headers(workbook_dict.get(constants.OSM, []),
                                              aliases.list_header,
                                              True)
    osm_tags = group_dictionaries_by_key(osm_sheet, constants.LIST_NAME)
    # #################################

    # Parse the survey sheet while generating a survey in our json format:

    row_number = 1  # We start at 1 because the column header row is not
    #                 included in the survey sheet (presumably).
    # A stack is used to keep track of begin/end expressions
    stack = [(None, json_dict.get(constants.CHILDREN))]
    # If a group has a table-list appearance flag
    # this will be set to the name of the list
    table_list = None
    # For efficiency we compile all the regular expressions
    # that will be used to parse types:
    end_control_regex = re.compile(r"^(?P<end>end)(\s|_)(?P<type>(" +
                                   '|'.join(aliases.control.keys()) + r"))$")
    begin_control_regex = re.compile(r"^(?P<begin>begin)(\s|_)(?P<type>(" +
                                     '|'.join(aliases.control.keys()) +
                                     r"))( (over )?(?P<list_name>\S+))?$")
    select_regexp = re.compile(
        r"^(?P<select_command>(" + '|'.join(aliases.select.keys()) +
        r")) (?P<list_name>\S+)" +
        "( (?P<specify_other>(or specify other|or_other|or other)))?$")
    cascading_regexp = re.compile(
        r"^(?P<cascading_command>(" +
        '|'.join(aliases.cascading.keys()) +
        r")) (?P<cascading_level>\S+)?$")
    osm_regexp = re.compile(
        r"(?P<osm_command>(" + '|'.join(aliases.osm.keys()) +
        ')) (?P<list_name>\S+)')

    # Rows from the survey sheet that should be nested in meta
    survey_meta = []

    for row in survey_sheet:
        row_number += 1
        prev_control_type, parent_children_array = stack[-1]
        # Disabled should probably be first
        # so the attributes below can be disabled.
        if u"disabled" in row:
            warnings.append(
                row_format_string % row_number +
                " The 'disabled' column header is not part of the current" +
                " spec. We recommend using relevant instead.")
            disabled = row.pop(u"disabled")
            if aliases.yes_no.get(disabled):
                continue

        # skip empty rows
        if len(row) == 0:
            continue

        # Get question type
        question_type = row.get(constants.TYPE)
        if not question_type:
            # if name and label are also missing,
            # then its a comment row, and we skip it with warning
            if not ((constants.NAME in row) or (constants.LABEL in row)):
                warnings.append(
                    row_format_string % row_number +
                    " Row without name, text, or label is being skipped:\n" +
                    str(row))
                continue
            raise PyXFormError(
                row_format_string % row_number +
                " Question with no type.\n" + str(row))

        # Pull out questions that will go in meta block
        if question_type == 'audit':
            # Force audit name to always be "audit" to follow XForms spec
            if 'name' in row and row['name'] not in [None, '', 'audit']:
                raise PyXFormError(row_format_string % row_number +
                    " Audits must always be named 'audit.'" +
                    " The name column should be left blank.")

            row['name'] = 'audit'
            survey_meta.append(row)
            continue

        if question_type == 'calculate':
            calculation = row.get('bind', {}).get('calculate')
            if not calculation:
                raise PyXFormError(
                    row_format_string % row_number + " Missing calculation.")

        # Check if the question is actually a setting specified
        # on the survey sheet
        settings_type = aliases.settings_header.get(question_type)
        if settings_type:
            json_dict[settings_type] = unicode(row.get(constants.NAME))
            continue

        # Try to parse question as a end control statement
        # (i.e. end loop/repeat/group):
        end_control_parse = end_control_regex.search(question_type)
        if end_control_parse:
            parse_dict = end_control_parse.groupdict()
            if parse_dict.get("end") and "type" in parse_dict:
                control_type = aliases.control[parse_dict["type"]]
                if prev_control_type != control_type or len(stack) == 1:
                    raise PyXFormError(
                        row_format_string % row_number +
                        " Unmatched end statement. Previous control type: " +
                        str(prev_control_type) +
                        ", Control type: " + str(control_type))
                stack.pop()
                table_list = None
                continue

        # Make sure the row has a valid name
        if constants.NAME not in row:
            if row['type'] == 'note':
                # autogenerate names for notes without them
                row['name'] = "generated_note_name_" + str(row_number)
            # elif 'group' in row['type'].lower():
            #     # autogenerate names for groups without them
            #     row['name'] = "generated_group_name_" + str(row_number)
            else:
                raise PyXFormError(row_format_string % row_number +
                                   " Question or group with no name.")
        question_name = unicode(row[constants.NAME])
        if not is_valid_xml_tag(question_name):
            error_message = row_format_string % row_number
            error_message += " Invalid question name [" + \
                             question_name.encode('utf-8') + "] "
            error_message += "Names must begin with a letter, colon,"\
                             + " or underscore."
            error_message += "Subsequent characters can include numbers," \
                             + " dashes, and periods."
            raise PyXFormError(error_message)

        if constants.LABEL not in row and \
           row.get(constants.MEDIA) is None and \
           question_type not in aliases.label_optional_types:
            # TODO: Should there be a default label?
            #      Not sure if we should throw warnings for groups...
            #      Warnings can be ignored so I'm not too concerned
            #      about false positives.
            warnings.append(
                row_format_string % row_number +
                " Question has no label: " + str(row))

        # Try to parse question as begin control statement
        # (i.e. begin loop/repeat/group):
        begin_control_parse = begin_control_regex.search(question_type)
        if begin_control_parse:
            parse_dict = begin_control_parse.groupdict()
            if parse_dict.get("begin") and "type" in parse_dict:
                # Create a new json dict with children, and the proper type,
                # and add it to parent_children_array in place of a question.
                # parent_children_array will then be set to its children array
                # (so following questions are nested under it)
                # until an end command is encountered.
                control_type = aliases.control[parse_dict["type"]]
                new_json_dict = row.copy()
                new_json_dict[constants.TYPE] = control_type
                child_list = list()
                new_json_dict[constants.CHILDREN] = child_list
                if control_type is constants.LOOP:
                    if not parse_dict.get("list_name"):
                        # TODO: Perhaps warn and make repeat into a group?
                        raise PyXFormError(
                            row_format_string % row_number +
                            " Repeat loop without list name.")
                    list_name = parse_dict["list_name"]
                    if list_name not in choices:
                        raise PyXFormError(
                            row_format_string % row_number +
                            " List name not in columns sheet: " + list_name)
                    new_json_dict[constants.COLUMNS] = choices[list_name]

                # Generate a new node for the jr:count column so
                # xpath expressions can be used.
                repeat_count_expression = new_json_dict.get(
                    'control', {}).get('jr:count')
                if repeat_count_expression:
                    generated_node_name = new_json_dict['name'] + "_count"
                    parent_children_array.append({
                        "name": generated_node_name,
                        "bind": {
                            "readonly": "true()",
                            "calculate": repeat_count_expression,
                        },
                        "type": "calculate",
                    })
                    new_json_dict['control']['jr:count'] = \
                        "${" + generated_node_name + "}"

                # Code to deal with table_list appearance flags
                # (for groups of selects)
                ctrl_ap = new_json_dict.get(u"control", {}).get(u"appearance")
                if ctrl_ap == constants.TABLE_LIST:
                    table_list = True
                    new_json_dict[u"control"][u"appearance"] = u"field-list"
                    # Generate a note label element so hints and labels
                    # work as expected in table-lists.
                    # see https://github.com/modilabs/pyxform/issues/62
                    if 'label' in new_json_dict or 'hint' in new_json_dict:
                        generated_label_element = {
                            "type": "note",
                            "name":
                                "generated_table_list_label_" + str(row_number)
                        }
                        if 'label' in new_json_dict:
                            generated_label_element[constants.LABEL] = \
                                new_json_dict[constants.LABEL]
                            del new_json_dict[constants.LABEL]
                        if 'hint' in new_json_dict:
                            generated_label_element['hint'] = \
                                new_json_dict['hint']
                            del new_json_dict['hint']
                        child_list.append(generated_label_element)
                if 'intent' in new_json_dict:
                    new_json_dict['control'] = \
                        new_json_dict.get(u"control", {})
                    new_json_dict['control']['intent'] = \
                        new_json_dict['intent']

                parent_children_array.append(new_json_dict)
                stack.append((control_type, child_list))
                continue

        # try to parse as a cascading select
        cascading_parse = cascading_regexp.search(question_type)
        if cascading_parse:
            parse_dict = cascading_parse.groupdict()
            if parse_dict.get("cascading_command"):
                cascading_level = parse_dict["cascading_level"]
                cascading_prefix = row.get(constants.NAME)
                if not cascading_prefix:
                    raise PyXFormError(
                        row_format_string % row_number +
                        " Cascading select needs a name.")
                # cascading_json = get_cascading_json(
                # cascading_choices, cascading_prefix, cascading_level)
                if len(cascading_choices) <= 0 or \
                   'questions' not in cascading_choices[0]:
                    raise PyXFormError(
                        "Found a cascading_select " + cascading_level +
                        ", but could not find " + cascading_level +
                        "in cascades sheet.")
                cascading_json = cascading_choices[0]['questions']
                json_dict['choices'] = choices
                include_bindings = False
                if 'bind' in row:
                    include_bindings = True
                for cq in cascading_json:
                    # include bindings
                    if include_bindings:
                        cq['bind'] = row['bind']

                    def replace_prefix(d, prefix):
                        for k, v in d.items():
                            if isinstance(v, basestring):
                                d[k] = v.replace('$PREFIX$', prefix)
                            elif isinstance(v, dict):
                                d[k] = replace_prefix(v, prefix)
                            elif isinstance(v, list):
                                d[k] = map(
                                    lambda x: replace_prefix(x, prefix), v)
                        return d

                    parent_children_array.append(
                        replace_prefix(cq, cascading_prefix))
                continue  # so the row isn't put in as is

        # Try to parse question as a select:
        select_parse = select_regexp.search(question_type)
        if select_parse:
            parse_dict = select_parse.groupdict()
            if parse_dict.get("select_command"):
                select_type = aliases.select[parse_dict["select_command"]]
                if select_type == 'select one external' \
                        and 'choice_filter' not in row:
                    warnings.append(
                        row_format_string % row_number +
                        u" select one external is only meant for"
                        u" filtered selects.")
                    select_type = aliases.select['select_one']
                list_name = parse_dict["list_name"]
                list_file_name, file_extension = os.path.splitext(list_name)

                if list_name not in choices \
                        and select_type != 'select one external' \
                        and file_extension not in ['.csv', '.xml']:
                    if not choices:
                        raise PyXFormError(
                            u"There should be a choices sheet in this xlsform."
                            u" Please ensure that the choices sheet name is "
                            u"all in small caps and has columns 'list name', "
                            u"'name', and 'label' (or aliased column names).")
                    raise PyXFormError(
                        row_format_string % row_number +
                        " List name not in choices sheet: " + list_name)

                # Validate select_multiple choice names by making sure
                # they have no spaces (will cause errors in exports).
                if select_type == constants.SELECT_ALL_THAT_APPLY \
                        and file_extension not in ['.csv', '.xml']:
                    for choice in choices[list_name]:
                        if ' ' in choice[constants.NAME]:
                            raise PyXFormError(
                                "Choice names with spaces cannot be added "
                                "to multiple choice selects. See [" +
                                choice[constants.NAME] + "] in [" +
                                list_name + "]")

                specify_other_question = None
                if parse_dict.get("specify_other") is not None:
                    select_type += u" or specify other"
                    # With this code we no longer need to handle or_other
                    # questions in survey builder.
                    # However, it depends on being able to use choice filters
                    # and xpath expressions that return empty sets.
                    # choices[list_name].append(
                    # {
                    #     'name': 'other',
                    #     'label': {default_language : 'Other'},
                    #     'orOther': 'true',
                    # })
                    # or_other_xpath = 'isNull(orOther)'
                    # if 'choice_filter' in row:
                    #   row['choice_filter'] += ' or ' + or_other_xpath
                    # else:
                    #   row['choice_filter'] = or_other_xpath

                    # specify_other_question = \
                    # {
                    #       'type':'text',
                    #       'name': row['name'] + '_specify_other',
                    #       'label':
                    #        'Specify Other for:\n"' + row['label'] + '"',
                    #       'bind' : {'relevant':
                    #                "selected(../%s, 'other')" % row['name']},
                    #     }

                new_json_dict = row.copy()
                new_json_dict[constants.TYPE] = select_type

                if row.get('choice_filter'):
                    if select_type == 'select one external':
                        new_json_dict['query'] = list_name
                    else:
                        new_json_dict['itemset'] = list_name
                        json_dict['choices'] = choices
                elif file_extension in ['.csv', '.xml']:
                    new_json_dict['itemset'] = list_name
                else:
                    new_json_dict[constants.CHOICES] = choices[list_name]

                # Code to deal with table_list appearance flags
                # (for groups of selects)
                if table_list is not None:
                    # Then this row is the first select in a table list
                    if not isinstance(table_list, basestring):
                        table_list = list_name
                        table_list_header = {
                            constants.TYPE: select_type,
                            constants.NAME:
                                "reserved_name_for_field_list_labels_" +
                                str(row_number),
                            # Adding row number for uniqueness # noqa
                            constants.CONTROL: {u"appearance": u"label"},
                            constants.CHOICES: choices[list_name],
                            # Do we care about filtered selects in table lists?
                            # 'itemset' : list_name,
                        }
                        parent_children_array.append(table_list_header)

                    if table_list != list_name:
                        error_message = row_format_string % row_number
                        error_message += " Badly formatted table list," \
                                         " list names don't match: " + \
                                         table_list + " vs. " + list_name
                        raise PyXFormError(error_message)

                    control = new_json_dict[u"control"] = \
                        new_json_dict.get(u"control", {})
                    control[u"appearance"] = "list-nolabel"
                parent_children_array.append(new_json_dict)
                if specify_other_question:
                    parent_children_array.append(specify_other_question)
                continue

        # Try to parse question as osm:
        osm_parse = osm_regexp.search(question_type)
        if osm_parse:
            parse_dict = osm_parse.groupdict()
            new_dict = row.copy()
            new_dict['type'] = constants.OSM

            if parse_dict.get('list_name') is not None:
                tags = osm_tags.get(parse_dict.get('list_name'))
                for tag in tags:
                    if osm_tags.get(tag.get('name')):
                        tag['choices'] = osm_tags.get(tag.get('name'))
                new_dict['tags'] = tags

            parent_children_array.append(new_dict)

            continue

        # range question_type
        if question_type == 'range':
            new_dict = process_range_question_type(row)
            parent_children_array.append(new_dict)
            continue

        # TODO: Consider adding some question_type validation here.

        # Put the row in the json dict as is:
        parent_children_array.append(row)

    if len(stack) != 1:
        raise PyXFormError("Unmatched begin statement: " + str(stack[-1][0]))

    if settings.get('flat', False):
        # print "Generating flattened instance..."
        add_flat_annotations(stack[0][1])

    meta_children = [] + survey_meta

    if aliases.yes_no.get(settings.get("omit_instanceID")):
        if settings.get("public_key"):
            raise PyXFormError(
                "Cannot omit instanceID, it is required for encryption.")
    else:
        # Automatically add an instanceID element:
        meta_children.append({
            "name": "instanceID",
            "bind": {
                "readonly": "true()",
                "calculate": settings.get(
                    "instance_id", "concat('uuid:', uuid())"),
            },
            "type": "calculate",
        })

    if 'instance_name' in settings:
        # Automatically add an instanceName element:
        meta_children.append({
            "name": "instanceName",
            "bind": {
                "calculate": settings['instance_name']
            },
            "type": "calculate",
        })

    if len(meta_children) > 0:
        meta_element = \
            {
                "name": "meta",
                "type": "group",
                "control": {
                    "bodyless": True
                },
                "children": meta_children
            }
        noop, survey_children_array = stack[0]
        survey_children_array.append(meta_element)

    # print_pyobj_to_json(json_dict)
    return json_dict


def parse_file_to_workbook_dict(path, file_object=None):
    """
    Given a xls or csv workbook file use xls2json_backends to create
    a python workbook_dict.
    workbook_dicts are organized as follows:
    {sheetname : [{column_header : column_value_in_array_indexed_row}]}
    """
    (filepath, filename) = os.path.split(path)
    if not filename:
        raise PyXFormError("No filename.")
    (shortname, extension) = os.path.splitext(filename)
    if not extension:
        raise PyXFormError("No extension.")

    if extension == ".xls" or extension == ".xlsx":
        return xls_to_dict(file_object if file_object is not None else path)
    elif extension == ".csv":
        return csv_to_dict(file_object if file_object is not None else path)
    else:
        raise PyXFormError("File was not recognized")


def get_filename(path):
    """
    Get the extensionless filename from a path
    """
    return os.path.splitext((os.path.basename(path)))[0]


def parse_file_to_json(path, default_name=None, default_language=u"default",
                       warnings=None, file_object=None):
    """
    A wrapper for workbook_to_json
    """
    if warnings is None:
        warnings = []
    workbook_dict = parse_file_to_workbook_dict(path, file_object)
    if default_name is None:
        default_name = unicode(get_filename(path))
    return workbook_to_json(
        workbook_dict, default_name, default_language, warnings)


def organize_by_values(dict_list, key):
    """
    dict_list -- a list of dicts
    key -- a key shared by all the dicts in dict_list
    Returns a dict of dicts keyed by the value of the specified key
    in each dictionary.
    If two dictionaries fall under the same key an error is thrown.
    If a dictionary is doesn't have the specified key it is omitted
    """
    result = {}
    for dicty in dict_list:
        if key in dicty:
            dicty_copy = dicty.copy()
            val = dicty_copy.pop(key)
            if val in result:
                raise Exception("Duplicate key: " + val)
            result[val] = dicty_copy
    return result


class SpreadsheetReader(object):
    def __init__(self, path_or_file):
        path = path_or_file
        try:
            path = path.name
        except AttributeError:
            pass
        self._dict = parse_file_to_workbook_dict(path)
        self._path = path
        self._id = unicode(get_filename(path))
        self._name = self._print_name = self._title = self._id

    def to_json_dict(self):
        return self._dict

    # TODO: Make sure the unicode chars don't show up
    def print_json_to_file(self, filename=""):
        if not filename:
            filename = self._path[:-4] + ".json"
        print_pyobj_to_json(self.to_json_dict(), filename)


class SurveyReader(SpreadsheetReader):
    """
    SurveyReader is a wrapper for the parse_file_to_json function.
    It allows us to use the old interface where a SpreadsheetReader
    based object is created then a to_json_dict function is called on it.
    """

    def __init__(self, path_or_file):
        if isinstance(path_or_file, basestring):
            self._file_object = None
            path = path_or_file
        else:
            self._file_object = path_or_file
            path = path_or_file.name

        self._warnings = []
        self._dict = parse_file_to_json(
            path, warnings=self._warnings, file_object=self._file_object)
        self._path = path

    def print_warning_log(self, warn_out_file):
        # Open file to print warning log to.
        warn_out = open(warn_out_file, 'w')
        warn_out.write('\n'.join(self._warnings))


class QuestionTypesReader(SpreadsheetReader):
    """
    Class for reading spreadsheet file that specifies the available
    question types.
    @see question_type_dictionary
    """

    def __init__(self, path):
        super(QuestionTypesReader, self).__init__(path)
        self._setup_question_types_dictionary()

    def _setup_question_types_dictionary(self):
        use_double_colons = has_double_colon(self._dict)
        types_sheet = u"question types"
        self._dict = self._dict[types_sheet]
        self._dict = dealias_and_group_headers(
            self._dict, {}, use_double_colons, u"default")
        self._dict = organize_by_values(self._dict, u"name")


# Not used internally (or on formhub)
# TODO: If this is used anywhere else it is probably broken
#      from the changes I made to SpreadsheetReader.
class VariableNameReader(SpreadsheetReader):
    def __init__(self, path):
        SpreadsheetReader.__init__(self, path)
        self._organize_renames()

    def _organize_renames(self):
        new_dict = {}
        variable_names_so_far = []
        assert u"Dictionary" in self._dict
        for d in self._dict[u"Dictionary"]:
            if u"Variable Name" in d:
                assert d[u"Variable Name"] not in variable_names_so_far, \
                    d[u"Variable Name"]
                variable_names_so_far.append(d[u"Variable Name"])
                new_dict[d[u"XPath"]] = d[u"Variable Name"]
            else:
                variable_names_so_far.append(d[u"XPath"])
        self._dict = new_dict


if __name__ == "__main__":
    # Open the excel file specified by the argument of this python call,
    # convert that file to json, then print it
    if len(sys.argv) < 2:
        # print "You must supply a file argument."
        filename = "xlsform_spec_test.xls"
        path = "/home/user/python-dev/xlsform/pyxform/tests/example_xls/"
        path += filename
    else:
        path = sys.argv[1]

    warnings = []
    json_dict = parse_file_to_json(path, warnings=warnings)
    print_pyobj_to_json(json_dict)

    if len(warnings) > 0:
        sys.stderr.write("Warnings:" + '\n')
        sys.stderr.write('\n'.join(warnings) + '\n')



#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class UnicodeStrings(PyxformTestCase):
    def test_unicode_snowman(self):
        self.assertPyxformXform(
            md="""
            | survey |      |         |       |
            |        | type | name    | label |
            |        | text | snowman | ☃     |
            """,
            errored=False,
            xml__contains=[
                '<label>☃</label>',
            ],
        )

    def test_smart_quotes(self):
        self.assertPyxformXform(
            ss_structure={
                'survey': [
                    {'type': 'select_one xyz',
                     'name': 'smart_single_quoted',
                     'label': u'''
                     ‘single-quoted’
                     '''.strip()},
                    {'type': 'text',
                     'name': 'smart_double_quoted',
                     'relevant': 'selected(${smart_single_quoted}, ‘xxx’)',
                     'label': u'''
                     “double-quoted”
                     '''.strip()},
                ],
                'choices': [
                    {'list_name': 'xyz',
                     'name': 'xxx',
                     'label': '‘Xxx’'},
                    {'list_name': 'xyz',
                     'name': 'yyy',
                     'label': '“Yyy”'},
                ],
                'settings': [{'version': 'q(‘-’)p'}],
            },
            errored=False,
            validate=False,
            name="quoth",
            xml__contains=[
                "'single-quoted",
                '"double-quoted"',
                "selected( /quoth/smart_single_quoted , 'xxx')",
                "<label>'Xxx'</label>",
                '<label>"Yyy"</label>',
                '''
                version="q('-')p"
                '''.strip(),
            ],
        )



from pyxform.tests_v1.pyxform_test_case import PyxformTestCase

class WhitespaceTest(PyxformTestCase):
    def test_over_trim(self):
        self.assertPyxformXform(
            name='issue96',
            md="""
            | survey  |                 |             |       |
            |         | type            | label       | name  |
            |         | text            | Ignored     | var   |
            |         | note            | ${var} text | label |
            """,
            xml__contains=[
                '<label><output value=" /issue96/var "/> text </label>',
            ])



"""
Testing simple cases for Xls2Json
"""
from unittest import TestCase
from pyxform.xls2json import SurveyReader
from pyxform.builder import create_survey_element_from_dict
from pyxform.tests import utils


class GroupTests(TestCase):

    def test_json(self):
        x = SurveyReader(utils.path_to_text_fixture("group.xls"))
        x_results = x.to_json_dict()
        expected_dict = {
            u'name': u'group',
            u'title': u'group',
            u'id_string': u'group',
            u'sms_keyword': u'group',
            u'default_language': u'default',
            u'type': u'survey',
            u'children': [
                {
                    u'name': u'family_name',
                    u'type': u'text',
                    u'label': {u'English': u"What's your family name?"}
                    },
                {
                    u'name': u'father',
                    u'type': u'group',
                    u'label': {u'English': u'Father'},
                    u'children': [
                        {
                            u'name': u'phone_number',
                            u'type': u'phone number',
                            u'label': {
                                u'English':
                                    u"What's your father's phone number?"}
                            },
                        {
                            u'name': u'age',
                            u'type': u'integer',
                            u'label': {u'English': u'How old is your father?'}
                            }
                        ],
                    },
                {
                    u'children': [
                        {
                            u'bind': {
                                'calculate': "concat('uuid:', uuid())",
                                'readonly': 'true()'
                            },
                            u'name': 'instanceID',
                            u'type': 'calculate'
                        }
                    ],
                    u'control': {
                        'bodyless': True
                    },
                    u'name': 'meta',
                    u'type': u'group'
                }
                ],
            }
        self.maxDiff = None
        self.assertEqual(x_results, expected_dict)

    def test_equality_of_to_dict(self):
        x = SurveyReader(utils.path_to_text_fixture("group.xls"))
        x_results = x.to_json_dict()

        survey = create_survey_element_from_dict(x_results)
        survey_dict = survey.to_json_dict()
        # using the builder sets the title attribute to equal name
        # this won't happen through reading the excel file as done by
        # SurveyReader.
        # Now it happens.
        # del survey_dict[u'title']
        self.maxDiff = None
        self.assertEqual(x_results, survey_dict)



from pyxform.odk_validate import _cleanup_errors

from unittest2 import TestCase
from pyxform.tests.utils import prep_class_config


class ODKValidateTests(TestCase):
    maxDiff = None

    @classmethod
    def setUpClass(cls):
        prep_class_config(cls=cls)

    def test_cleanup_error_message(self):
        test_str = self.config.get(
            self.cls_name, "test_cleanup_error_message_test")
        expected_str = self.config.get(
            self.cls_name, "test_cleanup_error_message_expected")
        self.assertEqual(_cleanup_errors(test_str), expected_str.strip())

    def test_do_not_over_trim_javarosa_errors(self):
        test_str = self.config.get(
            self.cls_name, "test_do_not_over_trim_javarosa_errors_test")
        # Unescape tabs in string
        test_str = test_str.replace("\n\\t", "\n\t")
        expected_str = self.config.get(
            self.cls_name, "test_do_not_over_trim_javarosa_errors_expected")
        self.assertEqual(_cleanup_errors(test_str), expected_str.strip())



from pyxform.xls2json import QuestionTypesReader, print_pyobj_to_json

FILE_MEDIA_TYPES = ",".join([
    "text/plain",
    "application/pdf",
    "application/vnd.ms-excel",
    "application/msword",
    "text/richtext",
    "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
    "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
    "application/zip",
    "application/x-zip",
    "application/x-zip-compressed"
])


def generate_new_dict():
    """
    This is just here incase there is ever any need to generate the question
    type dictionary from all.xls again.
    It shouldn't be called as part of any application.
    """
    path_to_question_types = "/home/nathan/aptana-workspace/pyxform"\
        "/pyxform/question_types/all.xls"
    json_dict = QuestionTypesReader(path_to_question_types).to_json_dict()
    print_pyobj_to_json(json_dict, 'new_quesiton_type_dict.json')


QUESTION_TYPE_DICT = \
    {
        "q picture": {
            "control": {
                "tag": "upload",
                "mediatype": "image/*"
            },
            "bind": {
                "type": "binary"
            }
        },
        "photo": {
            "control": {
                "tag": "upload",
                "mediatype": "image/*"
            },
            "bind": {
                "type": "binary"
            }
        },
        "add date time prompt": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "dateTime"
            }
        },
        "add audio prompt": {
            "control": {
                "tag": "upload",
                "mediatype": "audio/*"
            },
            "bind": {
                "type": "binary"
            }
        },
        "q date time": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "dateTime"
            }
        },
        "phonenumber": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "phonenumber"
            }
        },
        "get start time": {
            "bind": {
                "jr:preload": "timestamp",
                "type": "dateTime",
                "jr:preloadParams": "start"
            }
        },
        "add select multiple prompt using": {
            "control": {
                "tag": "select"
            },
            "bind": {
                "type": "select"
            }
        },
        "add note prompt": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "readonly": "true()",
                "type": "string"
            }
        },
        "calculate": {
            "bind": {
                "type": "string"
            }
        },
        "acknowledge": {
            "control": {
                "tag": "trigger"
            },
            "bind": {
                "type": "string"
            }
        },
        "location": {

            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "geopoint"
            }
        },
        "text": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "string"
            }
        },
        "select all that apply from": {
            "control": {
                "tag": "select"
            },
            "bind": {
                "type": "select"
            }
        },
        "simserial": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "simserial"
            }
        },
        "string": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "string"
            }
        },
        "q string": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "string"
            }
        },
        "imei": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "deviceid"
            }
        },
        "integer": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "int"
            }
        },
        "datetime": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "dateTime"
            }
        },
        "q note": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "readonly": "true()",
                "type": "string"
            }
        },
        "subscriber id": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "subscriberid"
            }
        },
        "decimal": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "decimal"
            }
        },
        "dateTime": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "dateTime"
            }
        },
        "q audio": {
            "control": {
                "tag": "upload",
                "mediatype": "audio/*"
            },
            "bind": {
                "type": "binary"
            }
        },
        "q geopoint": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "geopoint"
            }
        },
        "q geoshape": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "geoshape"
            }
        },
        "q geotrace": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "geotrace"
            }
        },
        "q image": {
            "control": {
                "tag": "upload",
                "mediatype": "image/*"
            },
            "bind": {
                "type": "binary"
            }
        },
        "get today": {
            "bind": {
                "jr:preload": "date",
                "type": "date",
                "jr:preloadParams": "today"
            }
        },
        "video": {
            "control": {
                "tag": "upload",
                "mediatype": "video/*"
            },
            "bind": {
                "type": "binary"
            }
        },
        "q acknowledge": {
            "control": {
                "tag": "trigger"
            },
            "bind": {
                "type": "string"
            }
        },
        "add video prompt": {
            "control": {
                "tag": "upload",
                "mediatype": "video/*"
            },
            "bind": {
                "type": "binary"
            }
        },
        "number of days in last month": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "int",
                "constraint": "0 <= . and . <= 31"
            },
            "hint": "Enter a number 0-31."
        },
        "get sim id": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "simserial"
            }
        },
        "q location": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "geopoint"
            }
        },
        "select one": {
            "control": {
                "tag": "select1"
            },
            "bind": {
                "type": "select1"
            }
        },
        "select one external": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "string"
            }
        },
        "add image prompt": {
            "control": {
                "tag": "upload",
                "mediatype": "image/*"
            },
            "bind": {
                "type": "binary"
            }
        },
        "select all that apply": {
            "control": {
                "tag": "select"
            },
            "bind": {
                "type": "select"
            }
        },
        "get end time": {
            "bind": {
                "jr:preload": "timestamp",
                "type": "dateTime",
                "jr:preloadParams": "end"
            }
        },
        "barcode": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "barcode"
            }
        },
        "q video": {
            "control": {
                "tag": "upload",
                "mediatype": "video/*"
            },
            "bind": {
                "type": "binary"
            }
        },
        "geopoint": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "geopoint"
            }
        },
        "geoshape": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "geoshape"
            }
        },
        "geotrace": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "geotrace"
            }
        },
        "select multiple from": {
            "control": {
                "tag": "select"
            },
            "bind": {
                "type": "select"
            }
        },
        "end time": {
            "bind": {
                "jr:preload": "timestamp",
                "type": "dateTime",
                "jr:preloadParams": "end"
            }
        },
        "device id": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "deviceid"
            }
        },
        "subscriberid": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "subscriberid"
            }
        },
        "q barcode": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "barcode"
            }
        },
        "q select": {
            "control": {
                "tag": "select"
            },
            "bind": {
                "type": "select"
            }
        },
        "select one using": {
            "control": {
                "tag": "select1"
            },
            "bind": {
                "type": "select1"
            }
        },
        "image": {
            "control": {
                "tag": "upload",
                "mediatype": "image/*"
            },
            "bind": {
                "type": "binary"
            }
        },
        "q int": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "int"
            }
        },
        "add text prompt": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "string"
            }
        },
        "add date prompt": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "date"
            }
        },
        "q calculate": {
            "bind": {
                "type": "string"
            }
        },
        "start": {
            "bind": {
                "jr:preload": "timestamp",
                "type": "dateTime",
                "jr:preloadParams": "start"
            }
        },
        "trigger": {
            "control": {
                "tag": "trigger"
            }
        },
        "add acknowledge prompt": {
            "control": {
                "tag": "trigger"
            },
            "bind": {
                "type": "string"
            }
        },
        "percentage": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "int",
                "constraint": "0 <= . and . <= 100"
            }
        },
        "get phone number": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "phonenumber"
            }
        },
        "today": {
            "bind": {
                "jr:preload": "date",
                "type": "date",
                "jr:preloadParams": "today"
            }
        },
        "gps": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "geopoint"
            }
        },
        "q date": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "date"
            }
        },
        "sim id": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "simserial"
            }
        },
        "add decimal prompt": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "decimal"
            }
        },
        "number of days in last six months": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "int",
                "constraint": "0 <= . and . <= 183"
            },
            "hint": "Enter a number 0-183."
        },
        "deviceid": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "deviceid"
            }
        },
        "int": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "int"
            }
        },
        "add barcode prompt": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "barcode"
            }
        },
        "select multiple using": {
            "control": {
                "tag": "select"
            },
            "bind": {
                "type": "select"
            }
        },
        "q decimal": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "decimal"
            }
        },
        "end": {
            "bind": {
                "jr:preload": "timestamp",
                "type": "dateTime",
                "jr:preloadParams": "end"
            }
        },
        "add calculate prompt": {
            "bind": {
                "type": "string"
            }
        },
        "add dateTime prompt": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "dateTime"
            }
        },
        "note": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "readonly": "true()",
                "type": "string"
            }
        },
        "add location prompt": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "geopoint"
            }
        },
        "get subscriber id": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "subscriberid"
            }
        },
        "phone number": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "string",
                "constraint": "regex(., '^\\d*$')"
            },
            "hint": "Enter numbers only."
        },
        "get device id": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "deviceid"
            }
        },
        "add integer prompt": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "int"
            }
        },
        "q dateTime": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "dateTime"
            }
        },
        "date": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "date"
            }
        },
        "q select1": {
            "control": {
                "tag": "select1"
            },
            "bind": {
                "type": "select1"
            }
        },
        "start time": {
            "bind": {
                "jr:preload": "timestamp",
                "type": "dateTime",
                "jr:preloadParams": "start"
            }
        },
        "number of days in last year": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "int",
                "constraint": "0 <= . and . <= 365"
            },
            "hint": "Enter a number 0-365."
        },
        "date time": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "dateTime"
            }
        },
        "time": {
            "control": {
                "tag": "input"
            },
            "bind": {
                "type": "time"
            }
        },
        "audio": {
            "control": {
                "tag": "upload",
                "mediatype": "audio/*"
            },
            "bind": {
                "type": "binary"
            }
        },
        "add select one prompt using": {
            "control": {
                "tag": "select1"
            },
            "bind": {
                "type": "select1"
            }
        },
        "hidden": {
            "bind": {
                "type": "string"
            }
        },
        "uri:subscriberid": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "uri:subscriberid"
            }
        },
        "uri:phonenumber": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "uri:phonenumber"
            }
        },
        "uri:simserial": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "uri:simserial"
            }
        },
        "uri:deviceid": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "uri:deviceid"
            }
        },
        "username": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "username"
            }
        },
        "uri:username": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "uri:username"
            }
        },
        "email": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "email"
            }
        },
        "uri:email": {
            "bind": {
                "jr:preload": "property",
                "type": "string",
                "jr:preloadParams": "uri:email"
            }
        },
        "osm": {
            "control": {
                "tag": "upload",
                "mediatype": "osm/*"
            },
            "bind": {
                "type": "binary"
            }
        },
        "file": {
            "control": {
                "tag": "upload",
                "mediatype": FILE_MEDIA_TYPES
            },
            "bind": {
                "type": "binary"
            }
        },
        "add file prompt": {
            "control": {
                "tag": "upload",
                "mediatype": FILE_MEDIA_TYPES
            },
            "bind": {
                "type": "binary"
            }
        },
        "range": {
            "control": {
                "tag": "range"
            },
            "bind": {
                "type": "int"
            }
        },
        "audit": {
            "bind": {
                "type": "binary"
            }
        }
    }



from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


MD = '''
| survey |           |           |                         |                             |                                                                                                                                                                                           |
|        | type      | name      | label                   | calculation                 | default                                                                                                                                                                                   |
|        | geoshape  | geoshape1 | Draw your shape here... |                             | 38.253094215699576 21.756382658677467;38.25021274773806 21.756382658677467;38.25007793942195 21.763892843919166;38.25290886154963 21.763935759263404;38.25146813817506 21.758421137528785 |
|        | calculate | result    |                         | enclosed-area(${geoshape1}) |                                                                                                                                                                                           |
'''  # nopep8

XML_CONTAINS = '''
<bind calculate="enclosed-area( /area/geoshape1 )" nodeset="/area/result" type="string"/>
'''.strip()  # nopep8


class AreaTest(PyxformTestCase):
    def test_area(self):
        self.assertPyxformXform(
            name="area",
            md=MD,
            xml__contains=[XML_CONTAINS],
            debug=True
        )



from unittest import TestCase
from pyxform.builder import create_survey_from_xls
from pyxform.tests import utils


class LoopTests(TestCase):
    def test_loop(self):
        path = utils.path_to_text_fixture('another_loop.xls')
        survey = create_survey_from_xls(path)
        self.maxDiff = None
        expected_dict = {
            u'name': u'another_loop',
            u'id_string': u'another_loop',
            u'sms_keyword': u'another_loop',
            u'default_language': u'default',
            u'title': u'another_loop',
            u'type': u'survey',
            u'children': [
                {
                    u'name': u'loop_vehicle_types',
                    u'type': u'group',
                    u'children': [
                        {
                            u'label': {u'English': u'Car',
                                       u'French': u'Voiture'},
                            u'name': u'car',
                            u'type': u'group',
                            u'children': [
                                {
                                    u'label': {
                                        u'English': u'How many do you have?',
                                        u'French': u'Combien avoir?'
                                    },
                                    u'name': u'total',
                                    u'type': u'integer'
                                },
                                {
                                    u'bind': {u'constraint': u'. <= ../total'},
                                    u'label': {
                                        u'English': u'How many are working?',
                                        u'French': u'Combien marcher?'
                                    },
                                    u'name': u'working',
                                    u'type': u'integer'
                                }
                            ],
                        },
                        {
                            u'label': {u'English': u'Motorcycle',
                                       u'French': u'Moto'},
                            u'name': u'motor_cycle',
                            u'type': u'group',
                            u'children': [
                                {
                                    u'label': {
                                        u'English': u'How many do you have?',
                                        u'French': u'Combien avoir?'
                                    },
                                    u'name': u'total',
                                    u'type': u'integer'
                                },
                                {
                                    u'bind': {u'constraint': u'. <= ../total'},
                                    u'label': {
                                        u'English': u'How many are working?',
                                        u'French': u'Combien marcher?'
                                    },
                                    u'name': u'working',
                                    u'type': u'integer'
                                }
                            ],
                        }]},
                {
                    u'children': [
                        {
                            u'bind': {
                                'calculate': "concat('uuid:', uuid())",
                                'readonly': 'true()'
                            },
                            u'name': 'instanceID',
                            u'type': 'calculate'
                        }
                    ],
                    u'control': {
                        'bodyless': True
                    },
                    u'name': 'meta',
                    u'type': u'group'
                }
            ],
        }
        self.assertEquals(survey.to_json_dict(), expected_dict)
